Imports System.Xml
Imports System.IO

Namespace DATA
    Public Class LanguageList
        'Variablen zur Speicherung der kompletten Liste und
        'des Standardeintrages

        'Enthlt alle mglilichen Language-Objekte aus der Language.xml
        Private Shared m_items As New ArrayList
        Public Shared ReadOnly Property Items(ByVal Index As Int16) As Language
            Get
                Return m_items(Index)
            End Get
        End Property

        Public Shared ReadOnly Property Items() As ArrayList
            Get
                Return m_items
            End Get
        End Property

        'Standard-Sprache zB Deutsch
        Private Shared m_standard As Language
        Public Shared Property Standard() As Language
            Get
                Return m_standard
            End Get
            Set(ByVal Value As Language)
                Settings.INI.Write("DVD", "Language", Value.Code)
                m_standard = Value
            End Set
        End Property

        'Original-Sprache zB Englisch
        Private Shared m_original As Language
        Public Shared Property Original() As Language
            Get
                Return m_original
            End Get
            Set(ByVal Value As Language)
                Settings.INI.Write("DVD", "Original Language", Value.Code)
                m_original = Value
            End Set
        End Property

        'Abrufen eines Language-Objekts dessen Code vorhanden ist
        Public Shared ReadOnly Property GetLanguageByCode(ByVal Code As String) As Language
            Get
                For Each Language As Data.Language In Items()
                    If Language.Code = Code Then
                        Return Language
                    End If
                Next
                Return Nothing
            End Get
        End Property

        Public Shared Sub Load()
            Dim tmpDocument As New XmlDocument
            Dim tmpMainNode As XmlElement
            Dim curNode As XmlNode

            'Zuerst wird der LanguageList ein Eintrag zugewiesen
            'welcher bei keiner Angabe einer Sprache verwendet wird
            Items.Add(New Language("***keine Sprache***", ""))

            'Es wird versucht die Standard- und Original-Sprache aus dem INI-File 
            'zu lesen sollte diesjedoch nicht mglich sein, dann wird keine
            'Sprache fr beide Werte gewhlt
            Dim stdCode As String
            Dim orgCode As String

            stdCode = Settings.INI.Read("DVD", "Language", "")

            If stdCode = "" Then
                Standard = Items(0)
                stdCode = Items(0).Code
            End If

            orgCode = Settings.INI.Read("DVD", "Original Language", "")

            If orgCode = "" Then
                Original = Items(0)
                orgCode = Items(0).Code
            End If

            'Wenn das XML-File nicht vorhanden ist, wird kein weiterer
            'Eintrag in der LanguageList vorgenommen
            Dim LanguageXML As String = Application.StartupPath & "/Language.xml"

            If File.Exists(LanguageXML) Then
                'Das Language.XML File wird geladen
                tmpDocument.Load(Application.StartupPath & "/Language.xml")
                tmpMainNode = tmpDocument.DocumentElement

                'Alle Eintrge werden der Reihe nach durchlaufen und der
                'LanguageList hinzugefgt
                For Each curNode In tmpMainNode
                    Dim curLang As Muxxi.DATA.Language
                    Dim Name As String
                    Dim Code As String

                    'Wenn ein Eintrag fehlerhaft sein sollte, dann wird der
                    'entsprechende XML-Knoten bersprungen und eine Fehler-
                    'meldung auf der Konsole ausgegeben
                    Try
                        Name = CStr(curNode.Attributes("Name").InnerText)
                        Code = CStr(curNode.Attributes("Code").InnerText)
                        curLang = New Language(Name, Code)

                        'Es wird geprft, ob der aktuelle Eintrag die Standard oder
                        'Original Sprache darstellt. Wenn dem so ist, werden die 
                        'Standard und Original Sprache mit dem richtigen Wert
                        'festgesetzt
                        If Code = stdCode Then Standard = curLang
                        If Code = orgCode Then Original = curLang

                        Items.Add(curLang)
                    Catch err As Exception
                        'Console.WriteLine(err.Message)
                    End Try
                Next
            End If

            Items.Sort()
        End Sub
    End Class

    'Klasse welche eine einzelne Sprache deklariert
    'Sie enthlt den Namen, welcher angezeigt wird
    'und den Lndercode, der fr die exakte Verarbeitung
    'in DVDauthor bentigt wird
    Public Class Language
        'IComparable wurde implementiert, um die Liste
        'nach dem Einlesen aufsteigend sortieren zu knnen
        Implements IComparable

        Private m_name As String
        Private m_code As String

        Sub New(ByVal Name As String, ByVal Code As String)
            Me.Name = Name
            Me.Code = Code
        End Sub

        Property Name() As String
            Get
                Return m_name
            End Get
            Set(ByVal Value As String)
                m_name = Value
            End Set
        End Property

        Property Code() As String
            Get
                Return m_code
            End Get
            Set(ByVal Value As String)
                m_code = Value
            End Set
        End Property

        Overridable Overloads Function CompareTo(ByVal obj As Object) As Integer Implements IComparable.CompareTo
            If TypeOf obj Is Language Then
                Return Me.Name().CompareTo(CType(obj, Language).Name())
            End If
        End Function
    End Class
End Namespace
