Imports System.IO
Imports System.Xml

Namespace Data
    Public Class Project
        Inherits ProjectViewItem

        'Auflistung der Movie Objekte
        Private m_movies As Data.MovieCollection
        Public ReadOnly Property Movies() As Data.MovieCollection
            Get
                Return Me.m_movies
            End Get
        End Property

        'Auflistung der Menu Objekte
        Private m_menus As Data.MenuCollection
        Public ReadOnly Property Menus() As Data.MenuCollection
            Get
                Return Me.m_menus
            End Get
        End Property

        'Einstellungen, die fr jedes Project individuell sein knnen

        'ffentliches Ereignis welches ein Update des ProjectListItems auslst
        Public Event TextChanged()

        Public Sub New()
            Initialize()
        End Sub

        Private Sub Initialize()
            Me.m_movies = New Data.MovieCollection(Me)
            Me.m_menus = New Data.MenuCollection(Me)
            Me.Text = String.Format("Projekt {0:yyyy-MM-dd HH:mm:ss}", Now)
        End Sub

        'Bezeichnung des Projekts
        Private m_text As String
        Public Shadows Property Text() As String
            Get
                Return Me.m_text
            End Get
            Set(ByVal Value As String)
                Me.m_text = Value
                RaiseEvent TextChanged()
            End Set
        End Property

        'Projekt-Typ (ES, MPG, DVD, VOB)
        Private m_type As Data.ProjectType = Settings.Output.Type
        Public Property Type() As Data.ProjectType
            Get
                Return Me.m_type
            End Get
            Set(ByVal Value As Data.ProjectType)
                Me.m_type = Value
                RaiseEvent TextChanged()
            End Set
        End Property

        'Ausgabeverzeichnis
        Private m_outputpath As String
        Public Property OutputPath() As String
            Get
                Return Me.m_outputpath
            End Get
            Set(ByVal Value As String)
                Me.m_outputpath = Value
                RaiseEvent TextChanged()
            End Set
        End Property


        'Soll eine vorhandene DVD-Struktur gelscht werden?
        Private m_keepdvdstructure As Boolean = Settings.DVD.KeepDVDStructure
        Public Property KeepDVDStructure() As Boolean
            Get
                Return Me.m_keepdvdstructure
            End Get
            Set(ByVal Value As Boolean)
                Me.m_keepdvdstructure = Value
            End Set
        End Property

        'Soll ein DVD-Men erstellt werden?
        Private m_createdvdmenu As Boolean
        Public Property CreateDVDMenu() As Boolean
            Get
                Return Me.m_createdvdmenu
            End Get
            Set(ByVal Value As Boolean)
                Me.m_createdvdmenu = Value
                Select Case Value
                    Case True
                        Me.Menus.Add(New Data.Menu("Menu1", "Menu1"))
                    Case False
                        Me.Menus.Clear()
                End Select
            End Set
        End Property

        Public ReadOnly Property Input() As String
            Get
                Dim Movie As Movie = Me.m_movies(0)
                Return Movie.InputFolder
            End Get
        End Property

        Public ReadOnly Property Temp() As String
            Get
                Select Case Settings.Temp.Use
                    Case True
                        Return Settings.Temp.Folder
                    Case False
                        Return Me.OutputPath
                End Select
            End Get
        End Property

        Public Sub SetOutput(ByVal Movie As Data.Movie)
            Dim OutputFolder As String

            Select Case Mid(Settings.Output.Folder, 2, 2) = ":\"
                Case True
                    Select Case Settings.Output.AutoSubfolder
                        Case True
                            OutputFolder = Settings.Output.Folder & "\" & Format(Now, "yyyy-MM-dd") & "-" & Movie.Text
                        Case False
                            OutputFolder = Settings.Output.Folder
                    End Select
                Case False
                    'Nicht erlaubte Zeichen werden aus dem Dateinamen entfernt
                    Settings.Output.Folder = Functions.GetLegalFilename(Settings.Output.Folder)

                    OutputFolder = Movie.InputFolder & "\" & Settings.Output.Folder
            End Select

            Me.OutputPath = OutputFolder
        End Sub



        Public Sub AddMovie(ByVal Movie As Movie)
            Movies.Add(Movie)
            RaiseEvent TextChanged()
        End Sub

#Region " Verarbeitung"
        Friend StartTime As Date

        Public Sub Muxx()
            StartTime = Now

            Dim Steps As Int16 = 0

            'Es werden die notwendigen Verzeichnisse angelegt
            'um anschlieend auch die LOG-Dateien anlegen zu knnen
            System.IO.Directory.CreateDirectory(Me.OutputPath)
            System.IO.Directory.CreateDirectory(Me.Temp)

            'Die LOG-Dateien werden festgelegt und initialisiert
            Dim LogText As String = Functions.GetLegalFilename(Me.Text)
            Select Case Settings.LOG.Folder
                Case Settings.LOG.SaveIn.Input
                    OUTPUT.ProjectProcessing.LOGFile = String.Format("{0}\{1}_Process.txt", Me.Input, LogText)
                    OUTPUT.LOG.LOGFile = String.Format("{0}\{1}_Muxxi.txt", Me.Input, LogText)
                Case Settings.LOG.SaveIn.Output
                    OUTPUT.ProjectProcessing.LOGFile = String.Format("{0}\{1}_Process.txt", Me.OutputPath, LogText)
                    OUTPUT.LOG.LOGFile = String.Format("{0}\{1}_Muxxi.txt", Me.OutputPath, LogText)
                Case Settings.LOG.SaveIn.Temp
                    OUTPUT.ProjectProcessing.LOGFile = String.Format("{0}\{1}_Process.txt", Me.Temp, LogText)
                    OUTPUT.LOG.LOGFile = String.Format("{0}\{1}_Muxxi.txt", Me.Temp, LogText)
            End Select

            OUTPUT.ProjectProcessing.Out = New StreamWriter(OUTPUT.ProjectProcessing.LOGFile, False, System.Text.Encoding.Default)
            OUTPUT.LOG.Out = New StreamWriter(OUTPUT.LOG.LOGFile, False, System.Text.Encoding.Default)

            OUTPUT.LOG.WriteLine(String.Format("Version: {0}", Settings.Programm.Name), Images.Icon.Info)
            OUTPUT.LOG.WriteLine(String.Format("Verarbeite Projekt: {0}", Me.Text), Images.Icon.MPG)

            Select Case Me.Type
                Case Data.ProjectType.ES
                    OUTPUT.LOG.WriteLine("Konvertiere PES zu ES Streams", Images.Icon.MPG)

                Case Data.ProjectType.MPG
                    OUTPUT.LOG.WriteLine("Erstelle MPG-Datei", Images.Icon.MPG)

                Case Data.ProjectType.VOB
                    OUTPUT.LOG.WriteLine("Erstelle einzelne VOB-Datei", Images.Icon.VOB)

                Case Data.ProjectType.DVD
                    OUTPUT.LOG.WriteLine(String.Format("Erstelle komplette DVD-Struktur aus {0:d} Titel", Me.Movies.Count), Images.Icon.DVD)
            End Select

            OUTPUT.LOG.EmptyLine()

            'Ausgabe Output-Verzeichnis im LOG-Dialog
            OUTPUT.LOG.WriteLine(String.Format("Ausgabe Pfad: {0}", Me.OutputPath), Images.Icon.Folder)

            'Ausgabe des Temp-Verzeichnis
            Select Case Settings.Temp.Use
                Case True
                    OUTPUT.LOG.WriteLine(String.Format("TEMP Pfad: {0}", Me.Temp), Images.Icon.Output)
                Case False
                    OUTPUT.LOG.WriteLine("TEMP Verzeichnis ausgeschaltet!", Images.Icon.Error)
            End Select

            OUTPUT.LOG.EmptyLine()

            'Prfung der Streams
            For Each Movie As Movie In Me.Movies
                'Es wird der BasisName im LOG-Dialog ausgegeben
                OUTPUT.LOG.WriteLine(String.Format("Titel {0:d} '{1}'", Me.Movies.IndexOf(Movie) + 1, Movie.Text), Images.Icon.Info)

                'Es wird geprft, ob ein JtG-LOG vorhanden ist
                Movie.JackLOG = Movie.InputFolder & "\OUTPUT.Log.txt"

                If Not File.Exists(Movie.JackLOG) Then
                    Movie.JackLOG = String.Empty
                End If

                'Wenn eine DVD-Struktur erstellt werden soll und
                'die EPG.txt kopiert werden soll, dann wird das nun durchgefhrt
                If Me.Type = Data.ProjectType.DVD AndAlso Settings.DVD.SaveEPGinOutput Then
                    Dim EPGFile As String = Movie.InputFolder & "\EPG.txt"
                    If File.Exists(EPGFile) Then
                        System.IO.File.Copy(EPGFile, String.Format("{0}\{1}_EPG.txt", Me.OutputPath, Movie.Text), True)
                    End If
                End If

                'Nun werden die Streams berprft
                'Wenn es sich um PES Streams handelt, dann werden diese in ES umgewandelt
                Select Case Movie.InputType
                    Case Data.InputType.PES
                        Call StartUdrec(Movie)
                        If OUTPUT.LOG.LogDialog.Status = ProcessStatusEnum.Abort OrElse OUTPUT.LOG.LogDialog.Status = ProcessStatusEnum.Error Then Exit Sub
                        If Movie.OnlyVideo Then Movie.GetAudioStreams(Me.Temp)
                    Case InputType.ES
                        If Movie.OnlyVideo Then Movie.GetAudioStreams(Me.Input)
                    Case InputType.PX_ES, InputType.PX_PES, InputType.PX_MPG
                        OUTPUT.LOG.WriteLine("Verwende durch Project-X erstellte ES-Dateien", Images.Icon.Info)
                        If Movie.OnlyVideo Then Movie.GetAudioStreams(Me.Input)
                End Select

                If Me.Type = Data.ProjectType.ES OrElse OUTPUT.LOG.LogDialog.Status = ProcessStatusEnum.Abort OrElse OUTPUT.LOG.LogDialog.Status = ProcessStatusEnum.Error Then Exit Sub

            Next

            OUTPUT.LOG.WriteLine("Folgende Dateien werden verarbeitet:", Images.Icon.Input)

            For Each Movie As Data.Movie In Me.Movies
                Select Case System.IO.Path.GetExtension(Movie.Video.FullPath)
                    Case ".mpv"
                        OUTPUT.LOG.WriteLine(String.Format("Titel {0:d}: {1}", Movies.IndexOf(Movie) + 1, Movie.Video.FileName), Images.Icon.Video)

                        For Each Audio As Data.Audio In Movie.Audios
                            Select Case System.IO.Path.GetExtension(Audio.FullPath)
                                Case ".ac3", ".mp2", ".mpa"
                                    OUTPUT.LOG.WriteLine(String.Format("Titel {0:d}: {1}: {2}", Movies.IndexOf(movie) + 1, Audio.Language.Name, Audio.FileName), Images.Icon.Audio)

                                Case Else
                                    OUTPUT.LOG.WriteLine(Audio.FileName, Images.Icon.Error)
                                    OUTPUT.LOG.LogDialog.Status = ProcessStatusEnum.Error
                            End Select
                        Next
                    Case Else
                        OUTPUT.LOG.WriteLine("Es wurde keine gltige Video-Datei gefunden!", Images.Icon.Error)
                        OUTPUT.LOG.WriteLine("Alle weiteren Verarbeitungsschritte werden daher abgebrochen", Images.Icon.None)
                        OUTPUT.LOG.LogDialog.Status = ProcessStatusEnum.Error
                End Select

                If OUTPUT.LOG.LogDialog.Status = ProcessStatusEnum.Abort OrElse OUTPUT.LOG.LogDialog.Status = ProcessStatusEnum.Error Then Exit Sub

                OUTPUT.LOG.EmptyLine()
            Next

            If Me.Type = Data.ProjectType.ES OrElse OUTPUT.LOG.LogDialog.Status = ProcessStatusEnum.Abort OrElse OUTPUT.LOG.LogDialog.Status = ProcessStatusEnum.Error Then Exit Sub

            Call StartMPLEX()

            If OUTPUT.LOG.LogDialog.Status = ProcessStatusEnum.Abort OrElse OUTPUT.LOG.LogDialog.Status = ProcessStatusEnum.Error Then Exit Sub

            OUTPUT.ProjectProcessing.Out.WriteLine("Mplex completed in " & Functions.EndTime(StartTime))
            If Me.Type = Data.ProjectType.MPG Then Exit Sub

            'DVDauthor starten und die Files erstellen lassen

            Dim OutputPath As String

            Select Case Me.Type
                Case Data.ProjectType.VOB
                    OutputPath = Me.Temp
                Case Data.ProjectType.DVD
                    OutputPath = Me.OutputPath
            End Select

            If Not Me.KeepDVDStructure Or Me.Type = Data.ProjectType.VOB Then
                OUTPUT.LOG.WriteLine("Lsche vorhandene VIDEO_TS und AUDIO_TS Verzeichnisse!", Images.Icon.Recycle)
                OUTPUT.LOG.EmptyLine()

                If Directory.Exists(OutputPath & "\VIDEO_TS") Then Directory.Delete(OutputPath & "\VIDEO_TS", True)
                If Directory.Exists(OutputPath & "\AUDIO_TS") Then Directory.Delete(OutputPath & "\AUDIO_TS", True)
            End If

            If Me.CreateDVDMenu AndAlso Me.Type = Data.ProjectType.DVD Then Call StartDVDMenuCreation()

            Call StartDVDauthor()

            If Me.Type = Data.ProjectType.DVD OrElse OUTPUT.LOG.LogDialog.Status = ProcessStatusEnum.Abort OrElse OUTPUT.LOG.LogDialog.Status = ProcessStatusEnum.Error Then Exit Sub

            'anschlieend werden alle VOB-Files des Directory-Object eingelesen
            OUTPUT.LOG.WriteLine("Starte VOB-Erstellung...", Images.Icon.Start)

            Call MergeVOBFiles()

            If OUTPUT.LOG.LogDialog.Status = ProcessStatusEnum.Abort OrElse OUTPUT.LOG.LogDialog.Status = ProcessStatusEnum.Error Then Exit Sub

            OUTPUT.ProjectProcessing.Out.WriteLine("Merging completed in " & Functions.EndTime(StartTime))
        End Sub


#Region " Udrec, MPLEX, DVDMenu, DVDauthor, VOB"
        Private Sub StartUdrec(ByVal Movie As Data.Movie)
            'Mit Udrec werden PES Streams in ES Streams umgewandelt
            OUTPUT.LOG.WriteLine("PES zu ES-Konvertierung wird gestartet...", Images.Icon.Start)

            OUTPUT.LOG.LogDialog.ProgressValue = 0

            Dim OutputPath As String
            Dim Arguments As String

            If Me.Type = Data.ProjectType.ES Then
                OutputPath = Me.OutputPath
            Else
                OutputPath = Me.Temp
            End If

            Dim UDBaseName As String = OutputPath & "\" & Movie.Text
            Dim Path As New System.IO.DirectoryInfo(Movie.Video.DirectoryName)
            Dim Files() As String

            Arguments = "-f """ & Movie.Video.FullPath & """"

            Select Case Movie.OnlyVideo
                Case True
                    Files = Functions.GetFiles(Path, "*.apes")

                    For Each File As String In Files
                        Arguments &= " """ & File & """"
                    Next
                Case False

                    For Each Audio As Data.Audio In Movie.Audios
                        Arguments &= " """ & Audio.FullPath & """"
                    Next
            End Select

            Arguments &= " -o """ & UDBaseName & """"

            If Not Movie.VSplit = -1 Then
                Arguments &= " -vsplit " & Movie.VSplit
                OUTPUT.LOG.WriteLine("Automatischer Video-Schnitt aktiviert! Erfolgt nach " & Movie.VSplit & " Sequenzen", Images.Icon.Info)
            End If

            If Not Movie.ASplit = -1 Then
                Arguments &= " -asplit " & Movie.ASplit
                OUTPUT.LOG.WriteLine("Automatischer Audio-Schnitt aktiviert! Erfolgt nach " & Movie.ASplit & " Sequenzen", Images.Icon.Info)
            End If

            If Settings.Udrec.RewriteGTC Then Arguments &= " -gtc"
            If Movie.GenerateIDD Then Arguments &= " -idd"
            If Settings.Udrec.UseMinSyncCount Then Arguments &= " -msc " & Settings.Udrec.MinSyncCount
            Arguments &= " -es"

            Dim ExistingFiles() As String = Functions.GetFiles(New System.IO.DirectoryInfo(OutputPath), String.Format("{0}*.MPV", Movie.Text))

            For Each FullName As String In ExistingFiles
                File.Delete(FullName)
            Next

            OUTPUT.LOG.LogDialog.ProgressMax = (New System.IO.FileInfo(Movie.Video.FullPath).Length) / (1024 ^ 2)

            Dim WatchMPV As New IO.FileSystemWatcher(OutputPath, String.Format("{0}*.MPV", Movie.Text))
            WatchMPV.EnableRaisingEvents = True
            WatchMPV.NotifyFilter = NotifyFilters.Size
            WatchMPV.SynchronizingObject = OUTPUT.LOG.LogDialog

            AddHandler WatchMPV.Changed, AddressOf MPV_Changed

            OUTPUT.ProjectProcessing.Start(Arguments, OUTPUT.ProjectProcessing.Type.udrec)

            WatchMPV.EnableRaisingEvents = False
            WatchMPV = Nothing

            If OUTPUT.LOG.LogDialog.Status = ProcessStatusEnum.Abort OrElse OUTPUT.LOG.LogDialog.Status = ProcessStatusEnum.Error Then Exit Sub

            OUTPUT.LOG.LogDialog.ProgressValue = 0

            OUTPUT.LOG.WriteLine("PES zu ES-Konvertierung abgeschlossen", Images.Icon.Complete)

            'Es werden alle neuen MPV Dateien durchlaufen um die Grte zu finden,
            'welche dann an die Settings bergeben wird
            Path = New System.IO.DirectoryInfo(OutputPath)
            Files = Functions.GetFiles(Path, String.Format("{0}*.mpv", Movie.Text))

            Dim Length As Int64
            Dim VideoFile As String

            For Each File As String In Files
                If Length = 0 Then
                    VideoFile = File
                    Length = New System.IO.FileInfo(File).Length
                ElseIf Length < New System.IO.FileInfo(File).Length Then
                    VideoFile = File
                    Length = New System.IO.FileInfo(File).Length
                End If
            Next

            Dim ratio As Double = Length / New System.IO.FileInfo(Movie.Video.FullPath).Length
            Movie.Video.FullPath = VideoFile

            If Not File.Exists(Movie.Video.FullPath) Then
                OUTPUT.LOG.WriteLine("Bei der Verarbeitung ist ein Fehler aufgetreten!", Images.Icon.Error)
                OUTPUT.LOG.LogDialog.Status = ProcessStatusEnum.Error
                Exit Sub
            End If

            If Movie.OnlyVideo Then Exit Sub

            'Es werden alle AC3 und MP2 Dateien des OutputVerzeichisses eingelesen
            'und die Dateigre mit jener der APES verglichen. Wenn die
            'Gre der Datei sich innerhalb einer 20%ingen Breite
            'befindet, dann wird der ursprngliche Pfad durch den neuen der
            'AC3 bzw. MP2 Datei ersetzt. Auserdem wird bei jedem Druchlauf der ersten
            'For Each Schleife der Dateityp berprft und die Verarbeitung nur
            'fortgesetzt, wenn es sich um eine APES Datei handelt. So wird
            'verhindert, dass eine Datei doppelt verarbeitet wird.
            'Zustzlich wird bei dieser Vorgehensweise angenommen,
            'dass sich die Reihenfolge der Dateien nicht gendert hat und
            'dadurch die Nummerierung durch die Umwandlung mit der ursprnglichen
            'Reihenfolge aus der ListView des StreamSelection-Dialogs bereinstimmt
            Dim ESLength As Int64
            Dim PESLength As Int64

            Dim SearchString As String = Left(Movie.Video.FileName, Movie.Video.FileName.Length - 6)

            Files = Functions.GetFiles(Path, SearchString & "??.ac3;" & SearchString & "??.mp2")

            For Each File As String In Files
                ESLength = New System.IO.FileInfo(File).Length / ratio

                For Each Audio As Data.Audio In Movie.Audios
                    If System.IO.Path.GetExtension(Audio.FullPath) = ".apes" Then
                        PESLength = New System.IO.FileInfo(Audio.FullPath).Length
                        If (PESLength * 0.85) < ESLength And (PESLength * 1.05) > ESLength Then
                            Audio.FullPath = File
                            Exit For
                        End If
                    End If
                Next
            Next
            OUTPUT.LOG.EmptyLine()
        End Sub

        Private Sub MPV_Changed(ByVal sender As Object, ByVal e As System.IO.FileSystemEventArgs)
            Dim FileName As String = Left(e.Name, e.Name.Length - 10)
            Dim Files As String() = Functions.GetFiles(New System.IO.FileInfo(e.FullPath).Directory, FileName & "*.mpv")
            Dim Size As Int16

            For Each File As String In Files
                Size += (New System.IO.FileInfo(File).Length) / (1024 ^ 2)
            Next
            OUTPUT.LOG.LogDialog.ProgressValue = Size
        End Sub

        Private Sub StartMPLEX()
            'Nun wird der Muxx-Vorgang gestartet
            Dim Arguments As String

            OUTPUT.LOG.EmptyLine()
            OUTPUT.LOG.WriteLine("Starte Mplex...", Images.Icon.Start)

            For Each Movie As Data.Movie In Me.Movies
                'Mit MPLEX ein fr DVDauthor verarbeitbares File erstellen
                'Es wird der BasisName im LOG-Dialog ausgegeben

                OUTPUT.LOG.LogDialog.ProgressValue = 0

                If Me.Movies.IndexOf(Movie) <> 0 Then OUTPUT.LOG.EmptyLine()

                OUTPUT.LOG.WriteLine(String.Format("Titel {0:d} '{1}'", Me.Movies.IndexOf(Movie) + 1, Movie.Text), Images.Icon.Info)

                Dim OutputPath As String

                If Me.Type = Data.ProjectType.MPG Then
                    OutputPath = Me.OutputPath
                Else
                    OutputPath = Me.Temp
                End If

                Arguments = "-r 0 -f 8 -o """ & OutputPath & "\" & Movie.Text & ".mpg"" """ & Movie.Video.FullPath & """ "

                For Each Audio As Data.Audio In Movie.Audios
                    Arguments &= """" & Audio.FullPath & """ "
                Next

                OUTPUT.LOG.WriteLine(String.Format("Erstelle '{0}.mpg'", Movie.Text), Images.Icon.Output)

                OUTPUT.LOG.LogDialog.ProgressMax = Movie.GetMovieSize

                Dim WatchMPG As New IO.FileSystemWatcher(OutputPath, String.Format("{0}.mpg", Movie.Text))
                WatchMPG.EnableRaisingEvents = True
                WatchMPG.NotifyFilter = NotifyFilters.Size
                WatchMPG.SynchronizingObject = OUTPUT.LOG.LogDialog

                AddHandler WatchMPG.Changed, AddressOf FileSize_Changed

                OUTPUT.ProjectProcessing.Start(Arguments, OUTPUT.ProjectProcessing.Type.mplex)

                WatchMPG.EnableRaisingEvents = False
                WatchMPG = Nothing

                If OUTPUT.LOG.LogDialog.Status = ProcessStatusEnum.Abort OrElse OUTPUT.LOG.LogDialog.Status = ProcessStatusEnum.Error Then Exit Sub

                If Movie.InputType = InputType.PES AndAlso Settings.Temp.DeleteES Then
                    Dim delFiles As String() = Functions.GetFiles(New System.IO.DirectoryInfo(Me.Temp), String.Format("{0}*.mpv;{0}*.mp2;{0}*.ac3;{0}*.idd", Movie.Text))
                    Dim File As String
                    For Each File In delFiles
                        System.IO.File.Delete(File)
                    Next

                    OUTPUT.LOG.WriteLine("ES-Dateien wurden gelscht!", Images.Icon.Info)
                End If

            Next
            OUTPUT.LOG.WriteLine("Muxx-Vorgang abgeschlossen", Images.Icon.Complete)
        End Sub

        Private Sub StartDVDMenuCreation()
            OUTPUT.LOG.EmptyLine()
            OUTPUT.LOG.WriteLine("Erstelle DVD-Titel Men...", Images.Icon.Start)

            OUTPUT.LOG.LogDialog.ProgressValue = 0
            OUTPUT.LOG.LogDialog.ProgressMax = 5

            Dim FileBackground As New Bitmap(720, 576)
            FileBackground.SetResolution(75, 80)

            Dim Background As Graphics
            'Die Anzeige wird in bester Qualitt durchgefhrt
            Background = Graphics.FromImage(FileBackground)

            Background.SmoothingMode = Drawing2D.SmoothingMode.HighQuality
            Background.TextRenderingHint = Drawing.Text.TextRenderingHint.AntiAliasGridFit
            Background.CompositingQuality = Drawing2D.CompositingQuality.HighQuality

            If Not Settings.DVDMenu.Background = String.Empty Then
                Dim BackImage As New Bitmap(Settings.DVDMenu.Background)
                BackImage = New Bitmap(BackImage, 720, 576)
                BackImage.SetResolution(75, 80)
                Background.DrawImage(BackImage, 0, 0)
                BackImage.Dispose()
            Else
                Background.FillRectangle(New SolidBrush(Settings.DVDMenu.BackgroundColor), 0, 0, 720, 576)
            End If

            'Das 'Hauptimage' fr die Buttons
            Dim FileImage As New Bitmap(720, 576)
            FileImage.SetResolution(75, 80)


            Dim Image As Graphics
            'Die Anzeige wird in bester Qualitt durchgefhrt
            Image = Graphics.FromImage(FileImage)
            Image.SmoothingMode = Drawing2D.SmoothingMode.HighQuality
            Image.TextRenderingHint = Drawing.Text.TextRenderingHint.AntiAliasGridFit
            Image.CompositingQuality = Drawing2D.CompositingQuality.HighQuality


            'Image zur Darstellung einer Markierung
            Dim FileHighlight As New Bitmap(720, 576)
            FileHighlight.SetResolution(75, 80)

            Dim Highlight As Graphics
            'Die Anzeige wird in bester Qualitt durchgefhrt
            Highlight = Graphics.FromImage(FileHighlight)
            Highlight.SmoothingMode = Drawing2D.SmoothingMode.HighQuality
            Highlight.TextRenderingHint = Drawing.Text.TextRenderingHint.AntiAliasGridFit
            Highlight.CompositingQuality = Drawing2D.CompositingQuality.HighQuality


            'Image zur Darstellung whrend der Auswahl
            Dim FileSelected As New Bitmap(720, 576)
            FileSelected.SetResolution(75, 80)

            Dim Selected As Graphics
            'Die Anzeige wird in bester Qualitt durchgefhrt
            Selected = Graphics.FromImage(FileSelected)
            Selected.SmoothingMode = Drawing2D.SmoothingMode.HighQuality
            Selected.TextRenderingHint = Drawing.Text.TextRenderingHint.AntiAliasGridFit
            Selected.CompositingQuality = Drawing2D.CompositingQuality.HighQuality


            Dim PosX As Int16 = 100
            Dim PosY As Int16 = 50
            Dim rect As RectangleF

            Select Case Settings.DVDMenu.ShowDVDTitle
                Case True
                    'berschrift zentriert zeichnen
                    rect = New RectangleF(50, PosY, 620, 100)

                    Dim sf As New StringFormat
                    sf.Alignment = StringAlignment.Center
                    sf.LineAlignment = StringAlignment.Center

                    Background.DrawString(Me.Text, New Font("Arial", 40, FontStyle.Bold, GraphicsUnit.Pixel), _
                        New SolidBrush(Settings.DVDMenu.TitleColor), rect, sf)

                    PosY = 200
                Case False
                    PosY = 100
            End Select

            Dim Buttons As New ArrayList

            For Each Movie As Movie In Me.Movies
                rect = New RectangleF(PosX, PosY, 520, 30)
                Dim ButtonSize As SizeF = Background.MeasureString(Movie.Text, New Font("Arial", 25, FontStyle.Bold, GraphicsUnit.Pixel))

                If Buttonsize.Width > 520 Then Buttonsize.Width = 520

                Background.DrawString(Movie.Text, New Font("Arial", 25, FontStyle.Bold, GraphicsUnit.Pixel), New SolidBrush(Settings.DVDMenu.ButtonColor), rect)
                Highlight.FillRectangle(New SolidBrush(Settings.DVDMenu.HighlightColor), PosX, PosY + 30, ButtonSize.Width, 5)
                Selected.FillRectangle(New SolidBrush(Settings.DVDMenu.SelectColor), PosX, PosY + 30, ButtonSize.Width, 5)

                Buttons.Add(ButtonSize)
                PosY += 50

                If PosY > 550 Then
                    OUTPUT.LOG.WriteLine("Men kann auf Grund der Filmanzahl nicht erstellt werden!", Images.Icon.Error)
                    Me.CreateDVDMenu = False
                    Exit Sub
                End If
            Next

            'Erstellen eines leeren Mens als Sprungmarke zu den Filmen
            Dim FileJumpMenu As New Bitmap(720, 576)
            FileJumpMenu.SetResolution(75, 80)

            Dim JumpMenu As Graphics
            'Die Anzeige wird in bester Qualitt durchgefhrt
            JumpMenu = Graphics.FromImage(FileJumpMenu)

            JumpMenu.SmoothingMode = Drawing2D.SmoothingMode.HighQuality
            JumpMenu.TextRenderingHint = Drawing.Text.TextRenderingHint.AntiAliasGridFit
            JumpMenu.CompositingQuality = Drawing2D.CompositingQuality.HighQuality

            JumpMenu.FillRectangle(Brushes.Black, 0, 0, 720, 576)



            FileBackground.Save(Me.Temp & "\Background.jpg", System.Drawing.Imaging.ImageFormat.Jpeg)
            FileImage.Save(Me.Temp & "\Image.png", System.Drawing.Imaging.ImageFormat.Png)
            FileHighlight.Save(Me.Temp & "\Highlight.png", System.Drawing.Imaging.ImageFormat.Png)
            FileSelected.Save(Me.Temp & "\Select.png", System.Drawing.Imaging.ImageFormat.Png)
            FileJumpMenu.Save(Me.Temp & "\JumpMenu.jpg", System.Drawing.Imaging.ImageFormat.Jpeg)

            FileBackground.Dispose()
            FileImage.Dispose()
            FileHighlight.Dispose()
            FileSelected.Dispose()
            FileJumpMenu.Dispose()

            OUTPUT.LOG.LogDialog.ProgressValue = 1



            Dim Arguments As String

            'JPEG2YUV fr das Titelmenu
            Arguments = "/C jpeg2yuv -n 1 -I p -f 25 -j """ & Me.Temp & "\Background.jpg"" > """ & Me.Temp & "\Background.yuv"""

            OUTPUT.ProjectProcessing.Start(Arguments, OUTPUT.ProjectProcessing.Type.jpeg2yuv)

            'JPEG2YUV fr das JumpMenu
            Arguments = "/C jpeg2yuv -n 1 -I p -f 25 -j """ & Me.Temp & "\JumpMenu.jpg"" > """ & Me.Temp & "\JumpMenu.yuv"""

            OUTPUT.ProjectProcessing.Start(Arguments, OUTPUT.ProjectProcessing.Type.jpeg2yuv)


            OUTPUT.LOG.LogDialog.ProgressValue = 2


            'MPEG2ENC fr das Titelmenu
            Arguments = "-n p -f 8 -o """ & Me.Temp & "\background.m2v"" """ & Me.Temp & "\background.yuv"""

            OUTPUT.ProjectProcessing.Start(Arguments, OUTPUT.ProjectProcessing.Type.mpeg2enc)

            'MPEG2ENC fr das JumpMenu
            Arguments = "-n p -f 8 -o """ & Me.Temp & "\JumpMenu.m2v"" """ & Me.Temp & "\JumpMenu.yuv"""

            OUTPUT.ProjectProcessing.Start(Arguments, OUTPUT.ProjectProcessing.Type.mpeg2enc)


            OUTPUT.LOG.LogDialog.ProgressValue = 3


            'MPLEX fr das Titelmenu
            Arguments = String.Format("-f 8 -o ""{0}\background.mpg"" ""{0}\Background.m2v""", Me.Temp)

            OUTPUT.ProjectProcessing.Start(Arguments, OUTPUT.ProjectProcessing.Type.mplex)

            'MPLEX fr das JumpMenu
            Arguments = String.Format("-f 8 -o ""{0}\JumpMenu.mpg"" ""{0}\JumpMenu.m2v""", Me.Temp)

            OUTPUT.ProjectProcessing.Start(Arguments, OUTPUT.ProjectProcessing.Type.mplex)

            OUTPUT.LOG.LogDialog.ProgressValue = 4


            'SPUMUX
            OUTPUT.LOG.WriteLine("Erstelle 'spumux.xml'", Images.Icon.Write)

            Dim Writer As New XmlTextWriter(Me.Temp & "\spumux.xml", System.Text.Encoding.Default)

            With Writer
                .Formatting = Formatting.Indented
                .Indentation = 4 'Bestimmt die Einrckung jedes Elementes
                .WriteStartDocument()
                .WriteStartElement("subpictures")
                .WriteStartElement("stream")
                .WriteStartElement("spu")
                .WriteAttributeString("start", "00:00:00.00")
                .WriteAttributeString("image", Me.Temp & "\image.png")
                .WriteAttributeString("highlight", Me.Temp & "\highlight.png")
                .WriteAttributeString("select", Me.Temp & "\select.png")
                .WriteAttributeString("autoorder", "rows")

                Dim count As Int16 = 1

                PosX = 100

                Select Case Settings.DVDMenu.ShowDVDTitle
                    Case True
                        PosY = 200
                    Case False
                        PosY = 100
                End Select

                For Each ButtonSize As SizeF In Buttons

                    .WriteStartElement("button")

                    Select Case Settings.DVD.UseDVDauthor_0_6_10
                        Case True
                            .WriteAttributeString("name", String.Format("Button {0}", count))
                        Case False
                            .WriteAttributeString("label", String.Format("Button {0}", count))
                    End Select

                    .WriteAttributeString("x0", PosX)
                    .WriteAttributeString("y0", PosY)
                    .WriteAttributeString("x1", PosX + buttonsize.width)
                    .WriteAttributeString("y1", PosY + 35)

                    count += 1
                    PosY += 50

                    If PosY > 550 Then
                        PosX += 100
                        PosY = 200
                    End If

                    .WriteEndElement() '</Button>
                Next

                .WriteEndElement() '</spu>
                .WriteEndElement() '</stream>
                .WriteEndElement() '</subpictures>

                .WriteEndDocument()
                .Close()
            End With

            Arguments = String.Format("/C spumux ""{0}\spumux.xml"" < ""{0}\Background.mpg"" > ""{0}\Menu.mpg""", Me.Temp)

            OUTPUT.ProjectProcessing.Start(Arguments, OUTPUT.ProjectProcessing.Type.spumux)

            OUTPUT.LOG.LogDialog.ProgressValue = 5

            OUTPUT.LOG.WriteLine("DVD-Titel Men erstellt...", Images.Icon.Complete)
        End Sub


        Private Sub StartDVDauthor()
            Dim Arguments As String

            OUTPUT.LOG.EmptyLine()
            OUTPUT.LOG.WriteLine("Starte DVD-Author...", Images.Icon.Start)
            OUTPUT.LOG.WriteLine("Erstelle 'dvdauthor.xml'", Images.Icon.Write)

            OUTPUT.LOG.LogDialog.ProgressValue = 0

            'Erstellung der XML-Config Datei fr DVDauthor
            'um eine vollwetige DVD-Struktur zu erhalten, welche
            'direkt auf DVD gebrannt werden kann

            Dim Writer As New XmlTextWriter(Me.Temp & "\dvdauthor.xml", System.Text.Encoding.Default)

            Dim OutputPath As String

            If Me.Type = Data.ProjectType.DVD Then
                OutputPath = Me.OutputPath
            Else
                OutputPath = Me.Temp
            End If

            With Writer
                .Formatting = Formatting.Indented
                .Indentation = 4 'Bestimmt die Einrckung jedes Elementes
                .WriteStartDocument()
                .WriteStartElement("dvdauthor")
                .WriteAttributeString("dest", OutputPath)
                .WriteStartElement("vmgm")

                'Erstellen des Titel-Mens
                If Me.CreateDVDMenu AndAlso Me.Type = Data.ProjectType.DVD Then
                    .WriteStartElement("menus")
                    .WriteStartElement("pgc")
                    'Der pre Befehl dient zur Vorauswahl eines Buttons. Im Register g2 wird
                    'der vorher verwendete Button gespeichert. Wenn dieser Wert kleiner als
                    '1024 ist (kein Button vorher markiert) dann wird automatisch der erste
                    'Button markiert. Ansonsten muss g2 durch 1024 teilbar sein, wenn das
                    'Register vorher richtig gesetzt wurde.
                    .WriteElementString("pre", "{ g1=0; if (g2 lt 1024) button=1024; if (g2 ge 1024) button=g2; }")

                    For Button As Int16 = 1 To Me.Movies.Count
                        .WriteStartElement("button")
                        .WriteAttributeString("name", String.Format("Button {0}", Button))
                        .WriteString("{g2=button;jump titleset " & Button & " menu entry root;}")
                        .WriteEndElement() '</button>
                    Next

                    .WriteStartElement("vob")
                    .WriteAttributeString("file", Me.Temp & "\Menu.mpg")
                    .WriteAttributeString("pause", "inf")
                    .WriteEndElement() '</vob>
                    .WriteEndElement() '</pgc>
                    .WriteEndElement() '</menus>
                End If

                .WriteEndElement() '</vmgm>

                For Each Movie As Data.Movie In Me.Movies
                    .WriteStartElement("titleset")

                    'Es wird ein Men erstellt, das direkt 
                    'auf den ersten Titel zeigt
                    If Me.CreateDVDMenu AndAlso Me.Type = Data.ProjectType.DVD Then
                        .WriteStartElement("menus")
                        .WriteStartElement("pgc")
                        .WriteAttributeString("entry", "root")

                        'ber den pre-Befehl wird das Register g1 geprft.
                        '0 = Das Men wurde von einem anderne Men aufgerufen -> Titel starten
                        '1 = Das Men wurde von einem Titel aufgerufen -> Zurck zum Titel-Men
                        '
                        'Dadurch erreicht man vom Titel aus durch Drcken der Tasten Men und
                        'Titel-Men auf einer Fernbedienung immer das Titel-Men. Auerdem wird das
                        'eigentliche Men garnicht aufgerufen. Sollte ein Player diese Funktion
                        'nicht untersttzen, greift der post-Befehl, welcher auf den Titel verweist.
                        .WriteElementString("pre", "{ if (g1 eq 0) jump title 1; if (g1 eq 1) jump vmgm menu entry title; }")

                        .WriteStartElement("vob")
                        .WriteAttributeString("file", Me.Temp & "\JumpMenu.mpg")
                        .WriteEndElement() '</vob>
                        .WriteElementString("post", "jump title 1;")
                        .WriteEndElement() '</pgc>
                        .WriteEndElement() '</menus>
                    End If

                    .WriteStartElement("titles")

                    'Fr jede vorhandene Audiospur wird die Sprache festgelegt
                    For Each Audio As Data.Audio In Movie.Audios
                        Dim Code As String = Audio.Language.Code.ToLower

                        .WriteStartElement("audio")
                        'Wenn keine Sprache gewhlt wurde, dann wird auch keine an DVDauthor bergegen
                        If Not Code = String.Empty Then .WriteAttributeString("lang", Code)
                        .WriteEndElement() '</AUDIO>
                    Next

                    'Es werden nur Kapitelgesetzt wenn als Kapitellnge nicht 0 angegeben wurde 
                    'und wenn der Output-Type eine DVD-Struktur sein soll
                    Dim Chapters As String = "0"

                    If Me.Type = Data.ProjectType.DVD Then
                        If Not Movie.ChapterDialog = String.Empty Then
                            Chapters = Movie.ChapterDialog

                        ElseIf Not Movie.ChapterLength = 0 Then
                            Dim srbChap As New System.Text.StringBuilder
                            srbChap.Append("0,")

                            For i As Int16 = Movie.ChapterLength To Movie.FilmLength Step Movie.ChapterLength
                                srbChap.Append(i & ":00,")
                            Next
                            srbChap.Remove(srbChap.Length - 1, 1)
                            Chapters = srbChap.ToString
                        End If
                    End If

                    .WriteStartElement("pgc")

                    If Me.CreateDVDMenu AndAlso Me.Type = Data.ProjectType.DVD Then
                        .WriteElementString("pre", "g1=1;")
                    End If

                    .WriteStartElement("vob")
                    .WriteAttributeString("file", Me.Temp & "\" & Movie.Text & ".mpg")
                    .WriteAttributeString("chapters", Chapters)

                    If Me.CreateDVDMenu AndAlso Me.Type = Data.ProjectType.DVD AndAlso Not Settings.DVDMenu.PauseAfterTitle = 0 Then
                        .WriteAttributeString("pause", Settings.DVDMenu.PauseAfterTitle) 'Nach dem Titel wird kurz pausiert
                    End If

                    .WriteEndElement() '</VOB>

                    If Me.CreateDVDMenu AndAlso Me.Type = Data.ProjectType.DVD Then
                        .WriteElementString("post", "call vmgm menu entry title;")
                    End If

                    .WriteEndElement() '</PGC>
                    .WriteEndElement() '</TITLES>
                    .WriteEndElement() '</TITLESET>
                Next

                .WriteEndElement()
                .WriteEndDocument()
                .Close()
            End With

            OUTPUT.LOG.WriteLine("'Dvdauthor.xml' erfolgreich erstellt", Images.Icon.Complete)
            OUTPUT.LOG.WriteLine("Generiere DVD-Struktur", Images.Icon.Complete)

            'Verarbeitung der erstellten XML-Config Datei durch DVDauthor
            Arguments = " -x """ & Me.Temp & "\dvdauthor.xml"""

            OUTPUT.ProjectProcessing.Start(Arguments, OUTPUT.ProjectProcessing.Type.dvdauthor)

            If OUTPUT.LOG.LogDialog.Status = ProcessStatusEnum.Abort OrElse OUTPUT.LOG.LogDialog.Status = ProcessStatusEnum.Error Then Exit Sub

            OUTPUT.LOG.WriteLine("DVD-Struktur erstellt", Images.Icon.Complete)
            OUTPUT.LOG.EmptyLine()

            If Settings.Temp.DeleteMPG Then
                For Each Movie As Movie In Me.Movies
                    System.IO.File.Delete(Me.Temp & "\" & Movie.Text & ".mpg")
                    OUTPUT.LOG.WriteLine(String.Format("'{0}' wurde gelscht!", Movie.Text & ".mpg"), Images.Icon.Info)
                Next
            End If
            OUTPUT.LOG.EmptyLine()
        End Sub

        Private Sub MergeVOBFiles()
            OUTPUT.LOG.LogDialog.ProgressValue = 0

            Dim Movie As Data.Movie = Me.Movies(0)
            Dim dirPathFiles As New System.IO.DirectoryInfo(Me.Temp & "\VIDEO_TS")

            Dim filSelFiles As System.IO.FileInfo() = dirPathFiles.GetFiles("*.vob")
            Dim curVob As System.IO.FileInfo

            Dim FStrNew, FStr As FileStream
            Dim BRead As BinaryReader
            Dim BWrNew As BinaryWriter

            OUTPUT.ProjectProcessing.Out.WriteLine()
            OUTPUT.ProjectProcessing.Out.WriteLine()
            OUTPUT.ProjectProcessing.Out.WriteLine()
            OUTPUT.ProjectProcessing.Out.WriteLine("------------------------------------------------------------------")
            OUTPUT.ProjectProcessing.Out.WriteLine("                         VOB Merge LOG")
            OUTPUT.ProjectProcessing.Out.WriteLine("------------------------------------------------------------------")
            OUTPUT.ProjectProcessing.Out.WriteLine()

            OUTPUT.LOG.WriteLine("Erstelle " & Movie.Text & ".vob Datei:", Images.Icon.Output)

            FStrNew = New FileStream(Me.OutputPath & "\" & Movie.Text & ".vob", FileMode.Create)
            BWrNew = New BinaryWriter(FStrNew)

            Dim Size As Int16

            For Each curVob In filSelFiles
                Size += curVob.Length / (1024 ^ 2)
            Next

            OUTPUT.LOG.LogDialog.ProgressMax = Size

            Dim WatchVOB As New IO.FileSystemWatcher(Me.OutputPath, String.Format("{0}.VOB", Movie.Text))
            WatchVOB.EnableRaisingEvents = True
            WatchVOB.NotifyFilter = NotifyFilters.Size
            WatchVOB.SynchronizingObject = OUTPUT.LOG.LogDialog

            AddHandler WatchVOB.Changed, AddressOf FileSize_Changed

            Dim i As Integer = 1

            For Each curVob In filSelFiles

                Dim strFilename As String = curVob.FullName
                OUTPUT.LOG.WriteLine(String.Format("verarbeite VOB#{0:d}: {1}", i, System.IO.Path.GetFileName(strFilename)), Images.Icon.Input)

                OUTPUT.ProjectProcessing.Out.WriteLine("Input File " & i & ": " & strFilename)
                i += 1

                FStr = New FileStream(strFilename, FileMode.Open)
                BRead = New BinaryReader(FStr)


                Dim data() As Byte

                Do
                    data = BRead.ReadBytes(4096 * 16)
                    BWrNew.Write(data)
                    Application.DoEvents()
                    If OUTPUT.LOG.LogDialog.Status = ProcessStatusEnum.Abort OrElse OUTPUT.LOG.LogDialog.Status = ProcessStatusEnum.Error Then
                        OUTPUT.LOG.WriteLine("VOB Erstellung abgebrochen!", Images.Icon.Error)
                        Exit Do
                    End If

                Loop While data.Length > 0

                BRead.Close()
            Next

            BWrNew.Close()

            WatchVOB.EnableRaisingEvents = False
            WatchVOB = Nothing

            If OUTPUT.LOG.LogDialog.Status = ProcessStatusEnum.Abort OrElse OUTPUT.LOG.LogDialog.Status = ProcessStatusEnum.Error Then Exit Sub

            OUTPUT.LOG.LogDialog.ProgressValue = 0

            OUTPUT.ProjectProcessing.Out.WriteLine()
            OUTPUT.ProjectProcessing.Out.WriteLine("Output: " & Me.OutputPath & "\" & Movie.Text & ".vob")

            OUTPUT.LOG.WriteLine("VOB-Datei erstellt", Images.Icon.Complete)

            If Settings.Temp.DeleteDVD Then
                System.IO.Directory.Delete(Me.Temp & "\VIDEO_TS", True)
                System.IO.Directory.Delete(Me.Temp & "\AUDIO_TS", True)

                OUTPUT.LOG.WriteLine("DVD-Struktur wurde gelscht!", Images.Icon.Info)
            End If
            OUTPUT.LOG.EmptyLine()
        End Sub

        Private Sub FileSize_Changed(ByVal sender As Object, ByVal e As System.IO.FileSystemEventArgs)
            OUTPUT.LOG.LogDialog.ProgressValue = (New System.IO.FileInfo(e.FullPath).Length) / (1024 ^ 2)
        End Sub

#End Region
#End Region
    End Class



    'Art des Inputtyps
    Public Enum InputType
        PES
        ES
        MPG
        PX_PES
        PX_ES
        PX_MPG
    End Enum

    'Art des Ausgabetyps
    Public Enum ProjectType
        ES
        MPG
        DVD
        VOB
    End Enum
End Namespace