Option Strict Off
Option Explicit On 

Class MPEG2Decoder
    Sub New()
    End Sub

    Sub New(ByVal FileName As String)
        Me.m_filename = FileName
        Initialize()
    End Sub

    Private Sub Initialize()
        If OpenMPEG2File(FileName, 0, -1) Then
            Call GetMPEG2FileInfo(m_videofileinfo)
            Call GetMPEG2FrameInfo(m_frameinfo) 'Read frame properties
            Call SetMPEG2PixelFormat(Me.m_pixelformat)

            'Scale frames to RGB color scaling
            Me.m_initialized = True
        Else
            Me.m_initialized = False
        End If
    End Sub

    Private m_pixelformat As PixelFormatType = PixelFormatType.VideoFormatRGB24
    Public Property PixelFormat() As PixelFormatType
        Get
            Return Me.m_pixelformat
        End Get
        Set(ByVal Value As PixelFormatType)
            Me.m_pixelformat = Value
            SetMPEG2PixelFormat(Value)
        End Set
    End Property

    Private m_filename As String
    Public Property FileName() As String
        Get
            Return Me.m_filename
        End Get
        Set(ByVal Value As String)
            Me.m_filename = Value
        End Set
    End Property

    Private m_initialized As Boolean = False
    Public ReadOnly Property IsInitialized() As Boolean
        Get
            Return Me.m_initialized
        End Get
    End Property

    Private m_frameinfo As FrameInfoType
    Public ReadOnly Property FPS() As Int16
        Get
            Return Me.m_frameinfo.FPS
        End Get
    End Property

    Public ReadOnly Property Size() As Size
        Get
            Return New System.Drawing.Size(Me.m_frameinfo.Width, Me.m_frameinfo.Height)
        End Get
    End Property

    Public ReadOnly Property AspectRatio() As String
        Get
            Return Me.m_aspectratio(Me.m_frameinfo.AspectRatio)
        End Get
    End Property

    Private m_picture As Bitmap = Nothing
    Public ReadOnly Property Picture() As Bitmap
        Get
            Return Me.m_picture
        End Get
    End Property

    Private m_videofileinfo As VideoFileInfoType

    Public ReadOnly Property FileSize() As Int64
        Get
            Return Me.m_videofileinfo.Size
        End Get
    End Property

    Public ReadOnly Property Position() As Int64
        Get
            Return Me.m_videofileinfo.Position
        End Get
    End Property

    Public Sub CloseMyMpegFile()
        CloseMPEG2File()
    End Sub

    Public Sub GoToPosition(ByRef bytepos As Int64)
        Call MPEG2Seek(bytepos) 'GoTo Fileposition in bytes
    End Sub

    Public Sub SkipFrames(ByVal Frames As Integer)
        Call SkipMPEG2Frames(Frames)
        DecodeFrame()
    End Sub

    Public Event FrameChanged(ByVal sender As Object)

    Public Sub DecodeFrame()
        Dim myFrame As IntPtr

        If Not Me.m_picture Is Nothing Then Me.m_picture.Dispose()

        Call GetMPEG2FrameInfo(m_frameinfo) 'Read frame properties
        Call GetMPEG2FileInfo(m_videofileinfo)

        myFrame = GetMPEG2Frame 'Get a pointer to the decoded bitmap form mpeg2lib

        m_picture = New System.Drawing.Bitmap(m_frameinfo.Width, m_frameinfo.Height, m_frameinfo.Width * 3, Imaging.PixelFormat.Format24bppRgb, myFrame)
        m_picture.RotateFlip(RotateFlipType.Rotate180FlipX)

        RaiseEvent FrameChanged(Me)
    End Sub

    Public Sub WriteFile(ByVal Size As Int64)
        Console.WriteLine(WriteDataToFile("TestMe.mpg", Size))
    End Sub

#Region "DLL-Deklarationen und Strukturen zur Kommunikation mit MPEG2Lib.dll"

    Private m_aspectratio As String() = {"", "1:1", "4:3", "16:9", "2.21:1"}

    Private Structure FrameInfoType ' This type is defined in mpeg2lib
        Dim Width As Integer
        Dim Height As Integer
        Dim FPS As Double
        Dim AspectRatio As Integer
    End Structure

    Private Structure VideoFileInfoType ' This type is defined in mpeg2lib
        Dim Size As Int64
        Dim Position As Int64
        Dim Frame As Integer
        Dim VideoPTS As Integer 'Timestamp in 1/90000 seconds
    End Structure

    Public Enum PixelFormatType
        VideoFormatRGB24 = 1       'RGB interleaved, default
        VideoFormatGray8 = 2       'Y plane
        VideoFormatYUV24Planar = 3 'YUV planes
    End Enum

    'Declares for mpeg2lib.dll
    Private Declare Function OpenMPEG2File Lib "mpeg2lib.dll" Alias "?OpenMPEG2File@@YGHPAD_J1@Z" (ByVal filename As String, ByVal offset As Int64, ByVal size As Int64) As Integer
    Private Declare Sub GetMPEG2FileInfo Lib "mpeg2lib.dll" Alias "?GetMPEG2FileInfo@@YGXPAUTVideoFileInfo@@@Z" (ByRef fileinfo As VideoFileInfoType)
    Private Declare Sub GetMPEG2FrameInfo Lib "mpeg2lib.dll" Alias "?GetMPEG2FrameInfo@@YGXPAUTVideoFrameInfo@@@Z" (ByRef frameinfo As FrameInfoType)
    Private Declare Function GetMPEG2Frame Lib "mpeg2lib.dll" Alias "?GetMPEG2Frame@@YGPAEXZ" () As IntPtr

    Private Declare Sub SkipMPEG2Frames Lib "mpeg2lib.dll" Alias "?SkipMPEG2Frames@@YGXH@Z" (ByVal frames As Integer)
    Private Declare Sub MPEG2Seek Lib "mpeg2lib.dll" Alias "?MPEG2Seek@@YGX_J@Z" (ByVal position As Int64)

    Private Declare Function WriteDataToFile Lib "mpeg2lib.dll" Alias "?WriteDataToFile@@YG_JPAD_J@Z" (ByVal filename As String, ByVal size As Decimal) As Decimal

    Private Declare Sub SetMPEG2PixelFormat Lib "mpeg2lib.dll" Alias "?SetMPEG2PixelFormat@@YGXH@Z" (ByVal PixelFormat As PixelFormatType)
    Private Declare Sub SetRGBScaleFlag Lib "mpeg2lib.dll" Alias "?SetRGBScaleFlag@@YGXH@Z" (ByVal colorscale As Integer)
    Private Declare Sub CloseMPEG2File Lib "mpeg2lib.dll" Alias "?CloseMPEG2File@@YGXXZ" ()
#End Region

End Class