Imports System
Imports System.ComponentModel
Imports System.Windows.Forms
Imports System.Drawing
Imports System.IO

Namespace UI
    Public Class ProjectList
        Inherits System.Windows.Forms.UserControl

        Public Sub New()
            MyBase.New()
        End Sub

        Private _items As New ArrayList
        Private _selecteditem As UI.ProjectListItem = Nothing

        Public Property Items() As ArrayList
            Get
                Return Me._items
            End Get
            Set(ByVal Value As ArrayList)
                Me._items = Value
                Me.Repaint()
                Me.SelectProject(Items(0))
            End Set
        End Property

        Public Property SelectedItem() As UI.ProjectListItem
            Get
                Return Me._selecteditem
            End Get
            Set(ByVal Value As UI.ProjectListItem)
                If Not Me.SelectedItem Is Nothing Then
                    Me.SelectedItem.Selected = False
                End If

                Me._selecteditem = Value

                If Not Value Is Nothing Then
                    Value.Selected = True

                    'Es wird die Position des markierten errechnet
                    'und gegebenenfalls dort hin gescrollt
                    If (Value.Location.Y + Value.Height) > Me.Height Then
                        Dim AutoScrollY As Integer = Value.Location.Y + Value.Height
                        AutoScrollY += Math.Abs(Me.AutoScrollPosition.Y)
                        AutoScrollY -= Me.Height
                        Me.AutoScrollPosition = New Drawing.Point(0, AutoScrollY)
                    ElseIf Value.Location.Y < 0 Then
                        Me.AutoScrollPosition = New Drawing.Point(0, -Value.Location.Y)
                    End If
                End If
            End Set
        End Property

        Public Sub Add()
            Dim Project As New Data.Project
            Project.AddMovie(New Muxxi.Data.Movie)
            Me.Items.Add(Project)
            Me.DrawItem(Project)
            Me.SelectProject(Project)
        End Sub

        Public Sub Add(ByVal Project As Data.Project)
            Me.Items.Add(Project)
            Me.DrawItem(Project)
        End Sub

        Public Sub DeleteSelected()
            Me.Items.Remove(Me.SelectedItem.Project)
            If Me.Items.Count = 0 Then
                Me.Add()
            End If

            Me.Repaint()
            Me.SelectProject(Me.Items(0))
        End Sub

        Private Sub Repaint()
            Me.Controls.Clear()

            For Each Project As Data.Project In Me.Items
                Me.DrawItem(Project)
            Next
        End Sub

        Private Sub DrawItem(ByVal Project As Data.Project)
            Dim Item As New UI.ProjectListItem(Project)
            Item.Size = New System.Drawing.Size(Me.Width - 17, 16)

            Select Case Me.Controls.Count
                Case 0
                    Item.Location = New System.Drawing.Point(0, 0)
                Case Else
                    Item.Location = New System.Drawing.Point(0, (Me.Controls.Count * 16) + Me.Controls(0).Location.Y)
            End Select

            Me.Controls.Add(Item)
            AddHandler Item.Click, AddressOf Item_Click
            AddHandler Item.DragDrop, AddressOf ProjectList_DragDrop
            AddHandler Item.DragOver, AddressOf ProjectList_DragOver
        End Sub

#Region " Klick-Ereigniss auf ein Item sowie dessen Verarbeitung und das Event 'SelectionChanged'"
        Public Event SelectionChanged()

        Private Sub Item_Click(ByVal sender As Object, ByVal e As System.EventArgs)
            If sender Is Nothing Then Exit Sub
            Dim Item As UI.ProjectListItem = DirectCast(sender, UI.ProjectListItem)

            If Not Me.SelectedItem Is Item Then
                Me.SelectedItem = Item
                RaiseEvent SelectionChanged()
            End If
        End Sub

        Private Sub SelectProject(ByVal Project As Data.Project)
            Dim Item As UI.ProjectListItem

            For Each Item In Me.Controls
                If Item.Project Is Project Then
                    Me.SelectedItem = Item
                    Exit For
                End If
            Next

            RaiseEvent SelectionChanged()
        End Sub
#End Region

#Region " bertragung der Drag und Drop Ereignisse auf die nchste Ebene"
        Public Shadows Event DragOver(ByVal sender As Object, ByVal e As System.Windows.Forms.DragEventArgs)

        Private Sub ProjectList_DragOver(ByVal sender As Object, ByVal e As System.Windows.Forms.DragEventArgs) Handles MyBase.DragOver
            RaiseEvent DragOver(sender, e)
        End Sub

        Public Shadows Event DragDrop(ByVal sender As Object, ByVal e As System.Windows.Forms.DragEventArgs)

        Private Sub ProjectList_DragDrop(ByVal sender As Object, ByVal e As System.Windows.Forms.DragEventArgs) Handles MyBase.DragDrop
            RaiseEvent DragDrop(sender, e)
        End Sub
#End Region

    End Class
End Namespace
