Imports System
Imports System.ComponentModel
Imports System.ComponentModel.Design.Serialization
Imports System.Collections
Imports System.Drawing
Imports System.Drawing.Design
Imports System.Globalization
Imports System.Reflection
Imports System.Windows.Forms

Public Class ProjectList
    Inherits Control


#Region " Constructor "

    Public Sub New()
        MyBase.New()
        Initialize()
    End Sub

    Friend WithEvents VScroll As VScrollBar

    Private Sub Initialize()
        Me.m_projects = New ProjectCollection(Me)

        VScroll = New VScrollBar

        VScroll.Minimum = 0
        VScroll.SmallChange = 1
        VScroll.Parent = Me
        VScroll.Hide()

        AddHandler VScroll.Scroll, AddressOf OnScroll

        m_font = New Font("Terminal", 8, FontStyle.Regular, GraphicsUnit.Point)

        SetStyle(ControlStyles.AllPaintingInWmPaint, True)
        SetStyle(ControlStyles.ContainerControl, True)
        SetStyle(ControlStyles.DoubleBuffer, True)
        SetStyle(ControlStyles.UserPaint, True)
        SetStyle(ControlStyles.ResizeRedraw, True)
        SetStyle(ControlStyles.Selectable, True)
        SetStyle(ControlStyles.Opaque, True)
        SetStyle(ControlStyles.UserMouse, True)
    End Sub

#End Region



#Region " Im Designer ausgeblendete Standard-Eigenschaften "
    <Browsable(False)> _
    Public Overrides Property BackgroundImage() As Image
        Get
        End Get
        Set(ByVal Value As Image)
        End Set
    End Property

#End Region



#Region " Properties & Fields "

    Private WithEvents m_projects As ProjectCollection
    <Browsable(False)> _
    ReadOnly Property Projects() As ProjectCollection
        Get
            Return Me.m_projects
        End Get
    End Property

    Public Event SelectionChanged(ByVal sender As Object, ByVal e As EventArgs)

    Private m_selectedproject As Data.Project
    Public Property SelectedProject() As Data.Project
        Get
            Return m_selectedproject
        End Get
        Set(ByVal Value As Data.Project)
            Me.m_selectedproject = Value
            Invalidate()
            RaiseEvent SelectionChanged(Me, New EventArgs)
        End Set
    End Property

    Private m_allowpaint As Boolean = True

#Region " ProjectTree Colors "
    Private m_forecolor As Color = Color.FromArgb(255, 0, 0, 0)
    <Category("Standard Farben"), DefaultValue(GetType(Color), "0, 0, 0")> _
    Shadows Property ForeColor() As Color
        Get
            Return m_forecolor
        End Get
        Set(ByVal Value As Color)
            If (Not (m_forecolor.Equals(Value))) Then
                m_forecolor = Value
                Invalidate()
            End If
        End Set
    End Property

    Private m_backcolor As Color = Color.FromArgb(255, 142, 179, 231)
    <Category("Standard Farben"), DefaultValue(GetType(Color), "142, 179, 231")> _
    Shadows Property BackColor() As Color
        Get
            Return m_backcolor
        End Get
        Set(ByVal Value As Color)
            If (Not (m_backcolor.Equals(Value))) Then
                m_backcolor = Value
                Invalidate()
            End If
        End Set
    End Property

    Private m_fadecolor As Color = Color.FromArgb(255, 217, 232, 252)
    <Category("Standard Farben"), DefaultValue(GetType(Color), "217, 232, 252")> _
    Property FadeColor() As Color
        Get
            Return m_fadecolor
        End Get
        Set(ByVal Value As Color)
            If (Not (m_fadecolor.Equals(Value))) Then
                m_fadecolor = Value
                Invalidate()
            End If
        End Set
    End Property

    Private m_bordercolor As Color = Color.FromArgb(255, 1, 45, 150)
    <Category("Standard Farben"), DefaultValue(GetType(Color), "1, 45, 150")> _
    Property BorderColor() As Color
        Get
            Return m_bordercolor
        End Get
        Set(ByVal Value As Color)
            If (Not (m_bordercolor.Equals(Value))) Then
                m_bordercolor = Value
                Invalidate()
            End If
        End Set
    End Property
#End Region

#Region " Selected Colors "
    Private m_selectedforecolor As Color = Color.FromArgb(255, 0, 0, 0)
    <Category("Selected Farben"), DefaultValue(GetType(Color), "0, 0, 0")> _
    Public Property SelectedForeColor() As Color
        Get
            Return m_selectedforecolor
        End Get
        Set(ByVal Value As Color)
            If (Not (m_selectedforecolor.Equals(Value))) Then
                m_selectedforecolor = Value
                Invalidate()
            End If
        End Set
    End Property

    Private m_selectedbackcolor As Color = Color.FromArgb(255, 255, 195, 67)
    <Category("Selected Farben"), DefaultValue(GetType(Color), "255, 195, 67")> _
    Public Property SelectedBackColor() As Color
        Get
            Return m_selectedbackcolor
        End Get
        Set(ByVal Value As Color)
            If (Not (m_selectedbackcolor.Equals(Value))) Then
                m_selectedbackcolor = Value
                Invalidate()
            End If
        End Set
    End Property

    Private m_selectedfadecolor As Color = Color.FromArgb(255, 255, 230, 178)
    <Category("Selected Farben"), DefaultValue(GetType(Color), "255, 230, 178")> _
    Public Property SelectedFadeColor() As Color
        Get
            Return m_selectedfadecolor
        End Get
        Set(ByVal Value As Color)
            If (Not (m_selectedfadecolor.Equals(Value))) Then
                m_selectedfadecolor = Value
                Invalidate()
            End If
        End Set
    End Property

    Private m_selectedbordercolor As Color = Color.FromArgb(255, 134, 158, 0)
    <Category("Selected Farben"), DefaultValue(GetType(Color), "134, 158, 0")> _
    Public Property SelectedBorderColor() As Color
        Get
            Return m_selectedbordercolor
        End Get
        Set(ByVal Value As Color)
            If (Not (m_selectedbordercolor.Equals(Value))) Then
                m_selectedbordercolor = Value
                Invalidate()
            End If
        End Set
    End Property
#End Region

#Region " Project Colors "
    Private m_projectforecolor As Color = Color.FromArgb(255, 255, 255, 255)
    <Category("Project Farben"), DefaultValue(GetType(Color), "255, 255, 255")> _
    Public Property ProjectForeColor() As Color
        Get
            Return m_projectforecolor
        End Get
        Set(ByVal Value As Color)
            If (Not (m_projectforecolor.Equals(Value))) Then
                m_projectforecolor = Value
                Invalidate()
            End If
        End Set
    End Property

    Private m_projectbackcolor As Color = Color.FromArgb(255, 9, 42, 127)
    <Category("Project Farben"), DefaultValue(GetType(Color), "9, 42, 127")> _
    Public Property ProjectBackColor() As Color
        Get
            Return m_projectbackcolor
        End Get
        Set(ByVal Value As Color)
            If (Not (m_projectbackcolor.Equals(Value))) Then
                m_projectbackcolor = Value
                Invalidate()
            End If
        End Set
    End Property

    Private m_projectfadecolor As Color = Color.FromArgb(255, 102, 145, 215)
    <Category("Project Farben"), DefaultValue(GetType(Color), "102, 145, 215")> _
    Public Property ProjectFadeColor() As Color
        Get
            Return m_projectfadecolor
        End Get
        Set(ByVal Value As Color)
            If (Not (m_projectfadecolor.Equals(Value))) Then
                m_projectfadecolor = Value
                Invalidate()
            End If
        End Set
    End Property

    Private m_projectbuttoncolor As Color = Color.FromArgb(255, 172, 191, 227)
    <Category("Project Farben"), DefaultValue(GetType(Color), "172, 191, 227")> _
    Public Property ProjectButtonColor() As Color
        Get
            Return m_projectbuttoncolor
        End Get
        Set(ByVal Value As Color)
            If (Not (m_projectbuttoncolor.Equals(Value))) Then
                m_projectbuttoncolor = Value
                Invalidate()
            End If
        End Set
    End Property
#End Region

    Private m_totalheight As Integer
    Protected Property TotalHeight() As Integer
        Get
            Return m_totalheight
        End Get
        Set(ByVal Value As Integer)
            m_totalheight = Value
        End Set
    End Property

    Private m_showprojects As Integer = 7
    <Category("Projekt"), DefaultValue(7)> _
    Protected Property ShowProjects() As Integer
        Get
            Return m_showprojects
        End Get
        Set(ByVal Value As Integer)
            If (Not (m_showprojects.Equals(Value))) Then
                m_showprojects = Value
                Invalidate()
            End If
        End Set
    End Property

    Private m_font As Font
    Public Overrides Property Font() As Font
        Get
            Return m_font
        End Get
        Set(ByVal Value As Font)
            m_font = Value
        End Set
    End Property

    Private m_itemheight As Integer = 16
    <Category("Settings"), DefaultValue(16)> _
    Public Property ItemHeight() As Integer
        Get
            Return m_itemheight
        End Get
        Set(ByVal Value As Integer)
            If (Not (m_itemheight.Equals(Value))) Then
                m_itemheight = Value
                Invalidate()
            End If
        End Set
    End Property


    Public ProjectFont As New Font("Terminal", 8, FontStyle.Regular, GraphicsUnit.Point)
    Public ProjectItemFont As New Font("Terminal", 8, FontStyle.Regular, GraphicsUnit.Point)
#End Region



#Region " Drawing "

    Public Sub BeginUpdate()
        Me.m_allowpaint = False
    End Sub

    Public Sub EndUpdate()
        Me.m_allowpaint = True
        Invalidate()
    End Sub

    Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        If Not m_allowpaint Then Exit Sub
        Dim g As Graphics = e.Graphics
        Dim r As Rectangle = ClientRectangle

        DrawBackground(g, r)
        DrawItems(g, r)

        SetupScrolls(r, m_totalheight)

        DrawBorder(g, r)
    End Sub

    Private Sub DrawBackground(ByVal g As Graphics, ByVal r As Rectangle)
        Dim gradient As New Drawing2D.LinearGradientBrush(ClientRectangle, BackColor, FadeColor, Drawing2D.LinearGradientMode.ForwardDiagonal)
        g.FillRectangle(gradient, ClientRectangle)
    End Sub

    Private Sub DrawItems(ByVal g As Graphics, ByVal r As Rectangle)
        Dim Y As Integer = ClientRectangle.X - VScroll.Value

        For Each Project As Data.Project In Me.m_projects
            If Project Is SelectedProject Then
                Project.SetBounds(0, 0, 0, False)
            Else
                Project.SetBounds(Y, Me.Width, ItemHeight, False)

                DrawProject2(g, Project)

                Y += ItemHeight + 1
            End If
        Next
        TotalHeight = Y - (ClientRectangle.X - VScroll.Value) + 1
    End Sub

    Private Sub DrawProject2(ByVal g As Graphics, ByVal Project As Data.Project)
        Dim drawFormat As New StringFormat
        drawFormat.FormatFlags = StringFormatFlags.NoWrap
        drawFormat.Trimming = StringTrimming.EllipsisCharacter

        Dim SelectArea As New Rectangle(Project.Bounds.X + 1, Project.Bounds.Y, Project.Bounds.Width - 3, Project.Bounds.Height)
        Dim Gradient As New Drawing2D.LinearGradientBrush(SelectArea, ProjectFadeColor, ProjectBackColor, Drawing2D.LinearGradientMode.Vertical)

        g.FillRectangle(Gradient, SelectArea)
        g.DrawString(Project.Text, Font, New SolidBrush(ProjectForeColor), 0, Project.Bounds.Y + 2, drawFormat)
        g.DrawString(Project.Type.ToString, Font, New SolidBrush(ProjectForeColor), Project.Bounds.Width - 100, Project.Bounds.Y + 1)
        g.DrawString(Project.Movies.Count, Font, New SolidBrush(ProjectForeColor), Project.Bounds.Width - 75, Project.Bounds.Y + 1)
        g.DrawRectangle(New Pen(BorderColor), SelectArea)
    End Sub

    Private Sub SetupScrolls(ByVal r As Rectangle, ByVal TotalHeight As Integer)
        Dim VVis As Boolean = VScroll.Visible
        If TotalHeight > r.Height Then
            VScroll.Top = r.Top + 1
            VScroll.Left = r.Right - VScroll.Width - 1
            VScroll.Maximum = TotalHeight - 2
            VScroll.LargeChange = r.Height
            VScroll.Height = r.Height - 2
            If VScroll.Value > (TotalHeight - r.Height) Then
                VScroll.Value = (TotalHeight - r.Height)
            End If
            VScroll.Show()
        Else
            VScroll.Hide()
            VScroll.Value = 0
        End If

        If (VVis <> VScroll.Visible) Then
            Invalidate()
        End If
    End Sub

    Private Sub DrawBorder(ByVal g As Graphics, ByVal r As Rectangle)
        ControlPaint.DrawBorder(g, r, BorderColor, ButtonBorderStyle.Solid)
    End Sub

#End Region



#Region " Events "

    Public Event Scroll(ByVal sender As Object, ByVal e As ScrollEventArgs)
    Protected Friend Sub OnScroll(ByVal sender As Object, ByVal e As ScrollEventArgs)
        Invalidate()
        RaiseEvent Scroll(sender, e)
    End Sub

    'Protected Overrides Sub OnMouseMove(ByVal e As System.Windows.Forms.MouseEventArgs)
    '    Dim curNode As ProjectTreeNode = Nothing

    '    For Each Movie As Data.Movie In m_project.Movies
    '        If Movie.IsMouseMove(e) Then
    '            curNode = Movie
    '            Exit For
    '        End If

    '        If Movie.Extended Then
    '            If Not Movie.Video.FullPath = String.Empty Then
    '                If Movie.Video.IsMouseMove(e) Then
    '                    curNode = Movie.Video
    '                    Exit For
    '                End If
    '            End If

    '            For Each Audio As Data.Audio In Movie.Audios
    '                If Audio.IsMouseMove(e) Then
    '                    curNode = Audio
    '                    Exit For
    '                End If
    '            Next
    '        End If
    '    Next
    '    If Not curNode Is Nothing Then Me.ToolTip.SetToolTip(Me, curNode.ToolTipText)

    'End Sub

    Protected Overrides Sub OnMouseDown(ByVal e As System.Windows.Forms.MouseEventArgs)
        MyBase.OnMouseDown(e)
        Console.WriteLine("MouseDown")

        For Each Project As Data.Project In Me.Projects
            If Project.IsMouseDown(e) Then
                Me.SelectedProject = Project
                Exit Sub
            End If
        Next
    End Sub

    Protected Overrides Sub OnMouseWheel(ByVal e As System.Windows.Forms.MouseEventArgs)
        MyBase.OnMouseWheel(e)
        If VScroll.Visible Then

            'Wenn sich die Scrollleiste am oberen Ende und weiter nach oben oder
            'die Scrollleiste ganz unten und weiter nach unten gescrollt wird,
            'dann wird der Vorgang abgebrochen, da ein Scrollen sinnlos wre
            If (VScroll.Value = VScroll.Minimum AndAlso e.Delta > 0) _
                OrElse (VScroll.Value = VScroll.Maximum AndAlso e.Delta < 0) Then Exit Sub

            Console.WriteLine("Min: {0}  Max: {1}  Val: {2}  Del: {3}", VScroll.Minimum, VScroll.Maximum, VScroll.Value, e.Delta)
            If ((VScroll.Value + -e.Delta) <= (VScroll.Maximum - VScroll.LargeChange)) AndAlso _
            (VScroll.Value + -e.Delta >= VScroll.Minimum) Then
                VScroll.Value += -e.Delta
                Invalidate()
            Else
                If -e.Delta < 0 Then
                    VScroll.Value = VScroll.Minimum
                    Invalidate()
                Else
                    VScroll.Value = (VScroll.Maximum - VScroll.LargeChange)
                    Invalidate()
                End If
            End If
        End If
    End Sub


#End Region


End Class

