Imports System
Imports System.ComponentModel
Imports System.ComponentModel.Design.Serialization
Imports System.Collections
Imports System.Drawing
Imports System.Drawing.Design
Imports System.Globalization
Imports System.Reflection
Imports System.Windows.Forms

Public Class ProjectStructure
    Inherits Control


#Region " Constructor "

    Public Sub New()
        MyBase.New()
        Initialize()
    End Sub

    Friend WithEvents StructureScroll As VScrollBar
    Friend WithEvents ListScroll As VScrollBar


    Private Sub Initialize()
        Me.m_projects = New ProjectCollection(Me)

        StructureScroll = New VScrollBar

        StructureScroll.Minimum = 0
        StructureScroll.SmallChange = 1
        StructureScroll.Parent = Me
        StructureScroll.Hide()

        AddHandler StructureScroll.Scroll, AddressOf OnScroll

        ListScroll = New VScrollBar

        ListScroll.Minimum = 0
        ListScroll.SmallChange = 1
        ListScroll.Parent = Me
        ListScroll.Hide()

        AddHandler ListScroll.Scroll, AddressOf OnScroll

        m_font = New Font("Terminal", 8, FontStyle.Regular, GraphicsUnit.Point)

        SetStyle(ControlStyles.AllPaintingInWmPaint, True)
        SetStyle(ControlStyles.ContainerControl, True)
        SetStyle(ControlStyles.DoubleBuffer, True)
        SetStyle(ControlStyles.UserPaint, True)
        SetStyle(ControlStyles.ResizeRedraw, True)
        SetStyle(ControlStyles.Selectable, True)
        SetStyle(ControlStyles.Opaque, True)
        SetStyle(ControlStyles.UserMouse, True)
    End Sub

#End Region



#Region " Im Designer ausgeblendete Standard-Eigenschaften "
    <Browsable(False)> _
    Public Overrides Property BackgroundImage() As Image
        Get
        End Get
        Set(ByVal Value As Image)
        End Set
    End Property

#End Region



#Region " Properties & Fields "

    Private WithEvents m_projects As ProjectCollection
    <Browsable(False)> _
    ReadOnly Property Projects() As ProjectCollection
        Get
            Return Me.m_projects
        End Get
    End Property

    Public Event SelectionChanged(ByVal sender As Object, ByVal e As EventArgs)

    Private m_selectedproject As Data.Project
    Public Property SelectedProject() As Data.Project
        Get
            Return m_selectedproject
        End Get
        Set(ByVal Value As Data.Project)
            Me.m_selectedproject = Value
            Invalidate()
            RaiseEvent SelectionChanged(Me, New EventArgs)
        End Set
    End Property


    Private m_selecteditem As ProjectViewItem = New ProjectViewItem
    <Browsable(False)> _
    Public Property SelectedItem() As ProjectViewItem
        Get
            Return m_selecteditem
        End Get
        Set(ByVal Value As ProjectViewItem)
            If (Not (m_selecteditem.Equals(Value))) Then
                m_selecteditem = Value
                Invalidate()
                RaiseEvent SelectionChanged(Me, New EventArgs)
            End If
        End Set
    End Property

    Private m_allowpaint As Boolean = True

#Region " ProjectTree Colors "
    Private m_forecolor As Color = Color.FromArgb(255, 0, 0, 0)
    <Category("Standard Farben"), DefaultValue(GetType(Color), "0, 0, 0")> _
    Shadows Property ForeColor() As Color
        Get
            Return m_forecolor
        End Get
        Set(ByVal Value As Color)
            If (Not (m_forecolor.Equals(Value))) Then
                m_forecolor = Value
                Invalidate()
            End If
        End Set
    End Property

    Private m_backcolor As Color = Color.FromArgb(255, 142, 179, 231)
    <Category("Standard Farben"), DefaultValue(GetType(Color), "142, 179, 231")> _
    Shadows Property BackColor() As Color
        Get
            Return m_backcolor
        End Get
        Set(ByVal Value As Color)
            If (Not (m_backcolor.Equals(Value))) Then
                m_backcolor = Value
                Invalidate()
            End If
        End Set
    End Property

    Private m_fadecolor As Color = Color.FromArgb(255, 217, 232, 252)
    <Category("Standard Farben"), DefaultValue(GetType(Color), "217, 232, 252")> _
    Property FadeColor() As Color
        Get
            Return m_fadecolor
        End Get
        Set(ByVal Value As Color)
            If (Not (m_fadecolor.Equals(Value))) Then
                m_fadecolor = Value
                Invalidate()
            End If
        End Set
    End Property

    Private m_bordercolor As Color = Color.FromArgb(255, 1, 45, 150)
    <Category("Standard Farben"), DefaultValue(GetType(Color), "1, 45, 150")> _
    Property BorderColor() As Color
        Get
            Return m_bordercolor
        End Get
        Set(ByVal Value As Color)
            If (Not (m_bordercolor.Equals(Value))) Then
                m_bordercolor = Value
                Invalidate()
            End If
        End Set
    End Property
#End Region

#Region " Selected Colors "
    Private m_selectedforecolor As Color = Color.FromArgb(255, 0, 0, 0)
    <Category("Selected Farben"), DefaultValue(GetType(Color), "0, 0, 0")> _
    Public Property SelectedForeColor() As Color
        Get
            Return m_selectedforecolor
        End Get
        Set(ByVal Value As Color)
            If (Not (m_selectedforecolor.Equals(Value))) Then
                m_selectedforecolor = Value
                Invalidate()
            End If
        End Set
    End Property

    Private m_selectedbackcolor As Color = Color.FromArgb(255, 255, 195, 67)
    <Category("Selected Farben"), DefaultValue(GetType(Color), "255, 195, 67")> _
    Public Property SelectedBackColor() As Color
        Get
            Return m_selectedbackcolor
        End Get
        Set(ByVal Value As Color)
            If (Not (m_selectedbackcolor.Equals(Value))) Then
                m_selectedbackcolor = Value
                Invalidate()
            End If
        End Set
    End Property

    Private m_selectedfadecolor As Color = Color.FromArgb(255, 255, 230, 178)
    <Category("Selected Farben"), DefaultValue(GetType(Color), "255, 230, 178")> _
    Public Property SelectedFadeColor() As Color
        Get
            Return m_selectedfadecolor
        End Get
        Set(ByVal Value As Color)
            If (Not (m_selectedfadecolor.Equals(Value))) Then
                m_selectedfadecolor = Value
                Invalidate()
            End If
        End Set
    End Property

    Private m_selectedbordercolor As Color = Color.FromArgb(255, 134, 158, 0)
    <Category("Selected Farben"), DefaultValue(GetType(Color), "134, 158, 0")> _
    Public Property SelectedBorderColor() As Color
        Get
            Return m_selectedbordercolor
        End Get
        Set(ByVal Value As Color)
            If (Not (m_selectedbordercolor.Equals(Value))) Then
                m_selectedbordercolor = Value
                Invalidate()
            End If
        End Set
    End Property
#End Region

#Region " Project Colors "
    Private m_projectforecolor As Color = Color.FromArgb(255, 255, 255, 255)
    <Category("Project Farben"), DefaultValue(GetType(Color), "255, 255, 255")> _
    Public Property ProjectForeColor() As Color
        Get
            Return m_projectforecolor
        End Get
        Set(ByVal Value As Color)
            If (Not (m_projectforecolor.Equals(Value))) Then
                m_projectforecolor = Value
                Invalidate()
            End If
        End Set
    End Property

    Private m_projectbackcolor As Color = Color.FromArgb(255, 9, 42, 127)
    <Category("Project Farben"), DefaultValue(GetType(Color), "9, 42, 127")> _
    Public Property ProjectBackColor() As Color
        Get
            Return m_projectbackcolor
        End Get
        Set(ByVal Value As Color)
            If (Not (m_projectbackcolor.Equals(Value))) Then
                m_projectbackcolor = Value
                Invalidate()
            End If
        End Set
    End Property

    Private m_projectfadecolor As Color = Color.FromArgb(255, 102, 145, 215)
    <Category("Project Farben"), DefaultValue(GetType(Color), "102, 145, 215")> _
    Public Property ProjectFadeColor() As Color
        Get
            Return m_projectfadecolor
        End Get
        Set(ByVal Value As Color)
            If (Not (m_projectfadecolor.Equals(Value))) Then
                m_projectfadecolor = Value
                Invalidate()
            End If
        End Set
    End Property

    Private m_projectbuttoncolor As Color = Color.FromArgb(255, 172, 191, 227)
    <Category("Project Farben"), DefaultValue(GetType(Color), "172, 191, 227")> _
    Public Property ProjectButtonColor() As Color
        Get
            Return m_projectbuttoncolor
        End Get
        Set(ByVal Value As Color)
            If (Not (m_projectbuttoncolor.Equals(Value))) Then
                m_projectbuttoncolor = Value
                Invalidate()
            End If
        End Set
    End Property
#End Region

    Private m_totalstructure As Integer
    Protected Property TotalStructure() As Integer
        Get
            Return m_totalstructure
        End Get
        Set(ByVal Value As Integer)
            m_totalstructure = Value
        End Set
    End Property

    Private m_totallist As Integer
    Protected Property TotalList() As Integer
        Get
            Return m_totallist
        End Get
        Set(ByVal Value As Integer)
            m_totallist = Value
        End Set
    End Property


    Private m_font As Font
    Public Overrides Property Font() As Font
        Get
            Return m_font
        End Get
        Set(ByVal Value As Font)
            m_font = Value
        End Set
    End Property

    Private m_showprojects As Integer = 3
    <Category("Projekt"), DefaultValue(3)> _
    Public Property ShowProjects() As Integer
        Get
            Return m_showprojects
        End Get
        Set(ByVal Value As Integer)
            If (Not (m_showprojects.Equals(Value))) Then
                m_showprojects = Value
                Invalidate()
            End If
        End Set
    End Property

    Private m_itemheight As Integer = 16
    <Category("Settings"), DefaultValue(16)> _
    Public Property ItemHeight() As Integer
        Get
            Return m_itemheight
        End Get
        Set(ByVal Value As Integer)
            If (Not (m_itemheight.Equals(Value))) Then
                m_itemheight = Value
                Invalidate()
            End If
        End Set
    End Property

    Private m_headerheight As Integer = 20
    <Category("Settings"), DefaultValue(20)> _
    Public Property HeaderHeight() As Integer
        Get
            Return m_headerheight
        End Get
        Set(ByVal Value As Integer)
            If (Not (m_headerheight.Equals(Value))) Then
                m_headerheight = Value
                Invalidate()
            End If
        End Set
    End Property


    Public ProjectFont As New Font("Terminal", 8, FontStyle.Regular, GraphicsUnit.Point)
    Public ProjectItemFont As New Font("Terminal", 8, FontStyle.Regular, GraphicsUnit.Point)
#End Region



#Region " Drawing "

    Public Sub BeginUpdate()
        Me.m_allowpaint = False
    End Sub

    Public Sub EndUpdate()
        Me.m_allowpaint = True
        Invalidate()
    End Sub

    Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        If Not m_allowpaint Then Exit Sub
        Dim g As Graphics = e.Graphics
        Dim r As Rectangle = ClientRectangle
        Dim structure_rect As New Rectangle(0, 0, Me.Width, Me.Height - Me.ListHeight)
        Dim list_rect As New Rectangle(0, Me.Height - Me.ListHeight, Me.Width, Me.ListHeight)

        DrawBackground(g, r)
        g.Clip = New Region(structure_rect)
        DrawStructure(g, structure_rect)
        SetupStructureScrolls(structure_rect, m_totalstructure)

        g.Clip = New Region(list_rect)
        DrawList(g, list_rect)
        SetupListScrolls(list_rect, m_totallist)

        g.Clip = New Region(r)
        DrawBorder(g, r)
    End Sub


    Private ReadOnly Property ListHeight() As Integer
        Get
            If Projects.Count <= Me.ShowProjects Then
                Return (Projects.Count - 1) * (ItemHeight + 1)
            Else
                Return Me.ShowProjects * (ItemHeight + 1)
            End If
        End Get
    End Property

    Private Sub DrawList(ByVal g As Graphics, ByVal r As Rectangle)
        Dim Y As Integer = r.Y - ListScroll.Value

        For Each Project As Data.Project In Me.m_projects
            If Not Project Is SelectedProject Then
                Project.SetBounds(Y, Me.Width, ItemHeight, False)

                DrawProject2(g, Project)

                Y += ItemHeight + 1
            End If
        Next
        TotalList = Y - (r.Y - ListScroll.Value)
    End Sub

    Private Sub DrawProject2(ByVal g As Graphics, ByVal Project As Data.Project)
        Dim drawFormat As New StringFormat
        drawFormat.FormatFlags = StringFormatFlags.NoWrap
        drawFormat.Trimming = StringTrimming.EllipsisCharacter

        Dim SelectArea As New Rectangle(Project.Bounds.X + 1, Project.Bounds.Y, Project.Bounds.Width - 3, Project.Bounds.Height)
        Dim Gradient As New Drawing2D.LinearGradientBrush(SelectArea, ProjectFadeColor, ProjectBackColor, Drawing2D.LinearGradientMode.Vertical)

        g.FillRectangle(Gradient, SelectArea)
        g.DrawString(Project.Text, Font, New SolidBrush(ProjectForeColor), 0, Project.Bounds.Y + 2, drawFormat)
        g.DrawString(Project.Type.ToString, Font, New SolidBrush(ProjectForeColor), Project.Bounds.Width - 100, Project.Bounds.Y + 1)
        g.DrawString(Project.Movies.Count, Font, New SolidBrush(ProjectForeColor), Project.Bounds.Width - 75, Project.Bounds.Y + 1)
        g.DrawRectangle(New Pen(BorderColor), SelectArea)
    End Sub

    Private Sub SetupListScrolls(ByVal r As Rectangle, ByVal TotalHeight As Integer)
        Dim VVis As Boolean = ListScroll.Visible
        If TotalHeight > r.Height Then
            ListScroll.Top = r.Top
            ListScroll.Left = r.Right - ListScroll.Width - 1
            ListScroll.Maximum = TotalHeight - 1
            ListScroll.LargeChange = r.Height
            ListScroll.Height = r.Height - 1
            If ListScroll.Value > (TotalHeight - r.Height) Then
                ListScroll.Value = (TotalHeight - r.Height)
            End If
            ListScroll.Show()
        Else
            ListScroll.Hide()
            ListScroll.Value = 0
        End If

        If (VVis <> ListScroll.Visible) Then
            Invalidate()
        End If
    End Sub



    Private Sub DrawBackground(ByVal g As Graphics, ByVal r As Rectangle)
        Dim gradient As New Drawing2D.LinearGradientBrush(ClientRectangle, BackColor, FadeColor, Drawing2D.LinearGradientMode.ForwardDiagonal)
        g.FillRectangle(gradient, ClientRectangle)
    End Sub

    Private Sub DrawStructure(ByVal g As Graphics, ByVal r As Rectangle)
        If SelectedProject Is Nothing Then Exit Sub
        Dim Y As Integer = ClientRectangle.X - StructureScroll.Value

        SelectedProject.SetBounds(0, Me.Width, 20, False)

        Y += 20 + 1

        For Each Menu As Data.Menu In SelectedProject.Menus
            Menu.SetBounds(Y, Me.Width, ItemHeight, Me.StructureScroll.Visible)

            DrawProjectItem(g, Menu)

            g.DrawString(Menu.Title, Font, Brushes.Black, ItemHeight, Y + 1)

            Y += ItemHeight
        Next

        For Each Movie As Data.Movie In SelectedProject.Movies
            Movie.SetBounds(Y, Me.Width, ItemHeight, Me.StructureScroll.Visible)

            DrawProjectItem(g, Movie)

            g.DrawString(Movie.Text, Font, Brushes.Black, ItemHeight, Y + 1)

            If Movie.Extended Then
                g.DrawImage(Images.Minus, 1, Y, 16, 16)
            Else
                g.DrawImage(Images.Plus, 1, Y, 16, 16)
            End If

            Y += ItemHeight

            If Movie.Extended Then
                If Not Movie.Video.FullPath = String.Empty Then
                    Movie.Video.SetBounds(Y, Me.Width, ItemHeight, Me.StructureScroll.Visible)

                    DrawProjectItem(g, Movie.Video)

                    g.DrawString(Movie.Video.ClipText(Movie.Video.FileName, Font, g, 250), Font, Brushes.Black, 32, Y + 1)
                    g.DrawString(Movie.Video.Infos.ToString, Font, Brushes.Black, 290, Y + 1)
                    g.DrawImage(Images.Video, ItemHeight, Y)
                    Y += ItemHeight
                End If

                For Each Audio As Data.Audio In Movie.Audios
                    Audio.SetBounds(Y, Me.Width, ItemHeight, Me.StructureScroll.Visible)
                    DrawProjectItem(g, Audio)

                    g.DrawString(Audio.ClipText(Audio.FileName, Font, g, 250), Font, Brushes.Black, 32, Y + 1)
                    g.DrawString(Audio.Infos.ToString, Font, Brushes.Black, 290, Y + 1)

                    Select Case Audio.Infos.Format
                        Case MultiMediaFile.AudioFormatType.AC3
                            g.DrawImage(Images.AC3, ItemHeight, Y)
                        Case MultiMediaFile.AudioFormatType.DTS
                            g.DrawImage(Images.Audio, ItemHeight, Y)
                        Case MultiMediaFile.AudioFormatType.MPA
                            Select Case Audio.Infos.Channels
                                Case "Mono"
                                    g.DrawImage(Images.Mono, ItemHeight, Y)
                                Case "Stereo", "Joint Stereo"
                                    g.DrawImage(Images.Stereo, ItemHeight, Y)
                                Case "Dual Channel"
                                    g.DrawImage(Images.DualChannel, ItemHeight, Y)
                            End Select
                        Case MultiMediaFile.AudioFormatType.WAV
                            g.DrawImage(Images.Audio, ItemHeight, Y)
                    End Select

                    Y += ItemHeight
                Next
            End If
        Next

        DrawProject(g, SelectedProject)

        TotalStructure = Y - (ClientRectangle.X - StructureScroll.Value) + 2
    End Sub

    Private Sub DrawProject(ByVal g As Graphics, ByVal Project As Data.Project)
        Dim gradbrush As Drawing2D.LinearGradientBrush

        Dim drawFormat As New StringFormat
        drawFormat.FormatFlags = StringFormatFlags.NoWrap
        drawFormat.Trimming = StringTrimming.EllipsisCharacter

        Dim DrawRectangle As New Rectangle(Project.Bounds.X, Project.Bounds.Y, Project.Bounds.Width, Project.Bounds.Height + 1)

        gradbrush = New Drawing2D.LinearGradientBrush(DrawRectangle, ProjectFadeColor, ProjectBackColor, Drawing2D.LinearGradientMode.Vertical)
        g.FillRectangle(gradbrush, DrawRectangle)
        g.DrawString(Project.Text, Font, New SolidBrush(ProjectForeColor), 0, Project.Bounds.Y + 4, drawFormat)
        g.DrawString(Project.Type.ToString, Font, New SolidBrush(ProjectForeColor), Project.Bounds.Width - 100, Project.Bounds.Y + 4)
        g.DrawString(Project.Movies.Count, Font, New SolidBrush(ProjectForeColor), Project.Bounds.Width - 75, Project.Bounds.Y + 4)
    End Sub

    Private Sub DrawProjectItem(ByVal g As Graphics, ByVal ProjectViewItem As ProjectViewItem)
        If ProjectViewItem Is SelectedItem Then
            Dim SelectArea As New Rectangle(ProjectViewItem.Bounds.X + 1, ProjectViewItem.Bounds.Y, ProjectViewItem.Bounds.Width - 3, ProjectViewItem.Bounds.Height)
            Dim Gradient As New Drawing2D.LinearGradientBrush(SelectArea, SelectedFadeColor, SelectedBackColor, Drawing2D.LinearGradientMode.Vertical)

            g.FillRectangle(Gradient, SelectArea)
            g.DrawRectangle(New Pen(SelectedBorderColor), SelectArea)
        End If
    End Sub

    Private Sub SetupStructureScrolls(ByVal r As Rectangle, ByVal TotalHeight As Integer)
        Dim VVis As Boolean = StructureScroll.Visible
        If TotalHeight > r.Height Then
            StructureScroll.Top = r.Top + HeaderHeight
            StructureScroll.Left = r.Right - StructureScroll.Width - 1
            StructureScroll.Maximum = TotalHeight - 1
            StructureScroll.LargeChange = r.Height
            StructureScroll.Height = r.Height - HeaderHeight
            If StructureScroll.Value > (TotalHeight - r.Height) Then
                StructureScroll.Value = (TotalHeight - r.Height)
            End If
            StructureScroll.Show()
        Else
            StructureScroll.Hide()
            StructureScroll.Value = 0
        End If

        If (VVis <> StructureScroll.Visible) Then
            Invalidate()
        End If
    End Sub

    Private Sub DrawBorder(ByVal g As Graphics, ByVal r As Rectangle)
        ControlPaint.DrawBorder(g, r, BorderColor, ButtonBorderStyle.Solid)
    End Sub

#End Region



#Region " Events "

    Public Event Scroll(ByVal sender As Object, ByVal e As ScrollEventArgs)
    Protected Friend Sub OnScroll(ByVal sender As Object, ByVal e As ScrollEventArgs)
        Invalidate()
        RaiseEvent Scroll(sender, e)
    End Sub

    Protected Overrides Sub OnMouseDown(ByVal e As System.Windows.Forms.MouseEventArgs)
        MyBase.OnMouseDown(e)

        For Each Menu As Data.Menu In Me.SelectedProject.Menus
            If Menu.IsMouseDown(e) Then
                Me.SelectedItem = Menu
                Exit Sub
            End If
        Next

        For Each Movie As Data.Movie In Me.SelectedProject.Movies
            If Movie.IsMouseDown(e) Then
                Me.SelectedItem = Movie
                Exit Sub
            End If

            If Movie.Extended Then
                If Not Movie.Video.FullPath = String.Empty Then
                    If Movie.Video.IsMouseDown(e) Then
                        Me.SelectedItem = Movie.Video
                        Exit Sub
                    End If
                End If

                For Each Audio As Data.Audio In Movie.Audios
                    If Audio.IsMouseDown(e) Then
                        Me.SelectedItem = Audio
                        Exit Sub
                    End If
                Next
            End If
        Next
    End Sub

    Protected Overrides Sub OnMouseWheel(ByVal e As System.Windows.Forms.MouseEventArgs)
        MyBase.OnMouseWheel(e)
        If StructureScroll.Visible Then

            'Wenn sich die Scrollleiste am oberen Ende und weiter nach oben oder
            'die Scrollleiste ganz unten und weiter nach unten gescrollt wird,
            'dann wird der Vorgang abgebrochen, da ein Scrollen sinnlos wre
            If (StructureScroll.Value = StructureScroll.Minimum AndAlso e.Delta > 0) _
                OrElse (StructureScroll.Value = StructureScroll.Maximum AndAlso e.Delta < 0) Then Exit Sub

            Console.WriteLine("Min: {0}  Max: {1}  Val: {2}  Del: {3}", StructureScroll.Minimum, StructureScroll.Maximum, StructureScroll.Value, e.Delta)
            If ((StructureScroll.Value + -e.Delta) <= (StructureScroll.Maximum - StructureScroll.LargeChange)) AndAlso _
            (StructureScroll.Value + -e.Delta >= StructureScroll.Minimum) Then
                StructureScroll.Value += -e.Delta
                Invalidate()
            Else
                If -e.Delta < 0 Then
                    StructureScroll.Value = StructureScroll.Minimum
                    Invalidate()
                Else
                    StructureScroll.Value = (StructureScroll.Maximum - StructureScroll.LargeChange)
                    Invalidate()
                End If
            End If
        End If
    End Sub


#End Region


End Class
