Namespace UI
    Public Class StreamTree
        Inherits TreeView

        'Events
        Public Event ActualizeOutput(ByVal Output As String)
        Public Event StreamCountChanged()

        'KontextMens
        Private ContextStream As ContextMenu
        Private ContextVideo As ContextMenu
        Private ContextAudio As ContextMenu
        Private ContextNone As ContextMenu
        Private WithEvents mniDelStream As New MenuItem("Lschen")
        Private WithEvents mniAddVideo As New MenuItem("Video")
        Private WithEvents mniAddAudio As New MenuItem("Audio")
        Private WithEvents mniRenameStream As New MenuItem("Umbenennen")

        Private WithEvents mniDelVideo As New MenuItem("Lschen")
        Private WithEvents mniDelAudio As New MenuItem("Lschen")
        Private WithEvents mniAudioLang As New MenuItem("Sprache")

        'Objekt um Zugriff auf das aktuelle Projekt zu erhalten
        Private _project As Data.Project

        Public Sub New()
            MyBase.New()

            InitializeComponents()
        End Sub

        Private Sub InitializeComponents()
            Me.ContextStream = New ContextMenu
            Me.ContextStream.MenuItems.AddRange(New MenuItem() {Me.mniRenameStream, Me.mniDelStream, mniAddVideo, mniAddAudio})

            Me.ContextVideo = New ContextMenu
            Me.ContextVideo.MenuItems.AddRange(New MenuItem() {Me.mniDelVideo})

            Me.ContextAudio = New ContextMenu
            Me.ContextAudio.MenuItems.AddRange(New MenuItem() {Me.mniDelAudio, Me.mniAudioLang})
        End Sub

        Public Sub Repaint(ByVal SelectStream As Muxxi.Data.Movie)
            Me.BeginUpdate()
            Me.Nodes.Clear()

            For Each Movie As Data.Movie In Project.Movies
                Dim StreamNode As New UI.StreamNode(Movie)
                Me.Nodes.Add(StreamNode)
                If Movie Is SelectStream Then Me.SelectedNode = StreamNode
            Next

            Me.ExpandAll()
            Me.EndUpdate()
        End Sub

        Public Sub MoveUp()
            If TypeOf Me.SelectedNode Is StreamNode Then
                Dim Movie As DATA.Movie = Me.SelectedMovie
                Dim Index As Int16 = Project.Movies.IndexOf(Movie)

                'Wenn es sich um das erste Element handelt, dann wird
                'der Vorgang abgebrochen
                If Index = 0 Then Exit Sub

                Project.Movies.RemoveAt(Index)
                Project.Movies.Insert(Index - 1, Movie)
                Me.Repaint(Movie)

            ElseIf TypeOf Me.SelectedNode Is AudioNode Then
                Dim Audio As Data.Audio = CType(Me.SelectedNode, AudioNode).Audio
                Dim Movie As DATA.Movie = Me.SelectedMovie
                Dim Index As Int16 = Movie.Audios.IndexOf(Audio)

                'Wenn es sich um das erste Element handelt, dann wird
                'der Vorgang abgebrochen
                If Index = 0 Then Exit Sub

                Movie.Audios.RemoveAt(Index)
                Movie.Audios.Insert(index - 1, Audio)
                Me.Repaint(Movie)
            End If
        End Sub

        Public Sub MoveDown()
            If TypeOf Me.SelectedNode Is StreamNode Then
                Dim Movie As DATA.Movie = Me.SelectedMovie
                Dim Index As Int16 = Project.Movies.IndexOf(Movie)

                'Wenn es sich um das letzte Element handelt, dann wird
                'der Vorgang abgebrochen
                If Index = (Project.Movies.Count - 1) Then Exit Sub

                Project.Movies.RemoveAt(Index)
                Project.Movies.Insert(Index + 1, Movie)
                Me.Repaint(Movie)

            ElseIf TypeOf Me.SelectedNode Is AudioNode Then
                Dim Audio As Data.Audio = CType(Me.SelectedNode, AudioNode).Audio
                Dim Movie As DATA.Movie = Me.SelectedMovie
                Dim Index As Int16 = Movie.Audios.IndexOf(Audio)

                'Wenn es sich um das erste Element handelt, dann wird
                'der Vorgang abgebrochen
                If Index = (Movie.Audios.Count - 1) Then Exit Sub

                Movie.Audios.RemoveAt(Index)
                Movie.Audios.Insert(index + 1, Audio)
                Me.Repaint(Movie)
            End If
        End Sub

#Region " Stream bezogene Methoden"
        Public Sub NewStream()
            Dim Movie As New Muxxi.Data.Movie

            Me.AddStream(Movie)
        End Sub

        Public Sub AddStream(ByVal Movie As DATA.Movie)
            Me.Project.AddMovie(Movie)
            Me.Repaint(Movie)
            RaiseEvent StreamCountChanged()
        End Sub

#Region " AddVideo"
        Public Sub AddVideo()
            Dim VideoFiles() As String = Me.ShowFileDialog(True)
            Dim VideoFile As String

            If Not VideoFiles Is Nothing Then
                For Each VideoFile In VideoFiles
                    Dim ScanFile As New MultiMediaFile.ScanFile(VideoFile)
                    If ScanFile.FileType = MultiMediaFile.FileType.Video Then
                        Me.AddVideo(VideoFile, ScanFile.VideoInfo, ScanFile.IsPES)
                    End If
                Next
            End If
        End Sub

        Public Sub AddVideo(ByVal VideoFile As String, ByVal VideoInfo As MultiMediaFile.VideoInfo, ByVal IsPES As Boolean)
            Me.SelectedMovie.SetVideoFile = VideoFile
            Me.SelectedMovie.Video.Infos = VideoInfo

            If Me.GetNodeCount(False) = 1 OrElse Me.Project.OutputPath = String.Empty Then
                Me.Project.SetOutput(Me.SelectedStreamNode.Movie)
                RaiseEvent ActualizeOutput(Me.Project.OutputPath)
            End If

            Select Case IsPES
                Case True
                    Me.SelectedMovie.InputType = DATA.InputType.PES
                Case False
                    Me.SelectedMovie.InputType = DATA.InputType.ES
            End Select

            Me.Repaint(Me.SelectedMovie)
        End Sub
#End Region
#Region " AddAudio"
        Public Sub AddAudio()
            Dim AudioFiles() As String = Me.ShowFileDialog(False)
            Dim AudioFile As String

            If Not AudioFiles Is Nothing Then
                For Each AudioFile In AudioFiles
                    Dim ScanFile As New MultiMediaFile.ScanFile(AudioFile)
                    If ScanFile.FileType = MultiMediaFile.FileType.Audio Then
                        Me.AddAudio(AudioFile, ScanFile.AudioInfo)
                    End If
                Next
            End If
        End Sub

        Public Sub AddAudio(ByVal AudioFile As String, ByVal AudioInfo As MultiMediaFile.AudioInfo)
            Dim Audio As Muxxi.Data.Audio = New Muxxi.Data.Audio(AudioFile, Muxxi.DATA.LanguageList.Standard)
            Audio.Infos = AudioInfo

            Me.SelectedMovie.AddAudio(Audio)
            Me.Repaint(Me.SelectedMovie)
        End Sub
#End Region

        Private Function ShowFileDialog(ByVal Video As Boolean) As String()
            Dim FileDialog As New OpenFileDialog

            Select Case Video
                Case True
                    FileDialog.Filter = "MPEG2 Video (*.mpv)|*.mpv|PES Video Stream (*.vpes)|*.vpes|All files (*.*)|*.*"
                    FileDialog.Multiselect = False
                Case False
                    FileDialog.Filter = "Audiodatei (*.mp2, *.mpa, *.ac3)|*.mp2; *.mpa; *.ac3|PES Audio Stream (*.apes)|*.apes|All files (*.*)|*.*"
                    FileDialog.Multiselect = True
            End Select

            Select Case Me.SelectedMovie.InputType
                Case DATA.InputType.ES
                    FileDialog.FilterIndex = 1
                Case DATA.InputType.PES
                    FileDialog.FilterIndex = 2
            End Select

            With FileDialog
                .RestoreDirectory = True
                .AddExtension = True
                .CheckFileExists = True
                .FileName = String.Empty
            End With

            If FileDialog.ShowDialog() = DialogResult.OK Then
                Return FileDialog.FileNames
            Else
                Return Nothing
            End If
        End Function

        Public Sub DeleteSelected()
            If TypeOf Me.SelectedNode Is UI.AudioNode Then
                Me.SelectedStreamNode.Movie.Audios.Remove(DirectCast(Me.SelectedNode, AudioNode).Audio)
            ElseIf TypeOf Me.SelectedNode Is UI.VideoNode Then
                Me.SelectedStreamNode.Movie.Video.FullPath = String.Empty
            ElseIf TypeOf Me.SelectedNode Is UI.StreamNode Then
                Me.Project.Movies.Remove(Me.SelectedMovie)
            End If

            Me.Nodes.Remove(Me.SelectedNode)

            If Me.Count = 0 Then
                Me.NewStream()
            End If
            RaiseEvent StreamCountChanged()
        End Sub
#End Region

#Region " Eigenschaften"
        Public ReadOnly Property SelectedStreamNode() As UI.StreamNode
            Get
                If Me.SelectedNode.Parent Is Nothing Then
                    Return DirectCast(Me.SelectedNode, UI.StreamNode)
                Else
                    Return DirectCast(Me.SelectedNode.Parent, UI.StreamNode)
                End If
            End Get
        End Property

        Public ReadOnly Property Count() As Int16
            Get
                Return Me.GetNodeCount(False)
            End Get
        End Property

        Public Property Project() As Data.Project
            Get
                Return Me._project
            End Get
            Set(ByVal Value As Data.Project)
                Me._project = Value
                If Not Value Is Nothing Then Me.Repaint(Project.Movies(0))
            End Set
        End Property

        Private ReadOnly Property SelectedMovie() As Muxxi.Data.Movie
            Get
                Return Me.SelectedStreamNode.Movie
            End Get
        End Property
#End Region

#Region " Ereignisse"
        Private Sub StreamTree_DoubleClick(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.DoubleClick
            Dim Node As TreeNode = Me.GetNodeAt(Me.PointToClient(Me.MousePosition()))

            If TypeOf (Node) Is UI.AudioNode Then
                Dim AudioNode As UI.AudioNode = DirectCast(Node, AudioNode)
                Dim dlgLanguage As New UI.LanguageDialog
                dlgLanguage.SelectedCode = AudioNode.Audio.Language

                If dlgLanguage.ShowDialog() = DialogResult.OK Then
                    AudioNode.Audio.Language = dlgLanguage.SelectedCode
                    AudioNode.Text = AudioNode.Audio.ToString
                End If
            ElseIf TypeOf Node Is UI.VideoNode Then
                Dim Preview As New UI.MPEG2preview
                Preview.SetFile(Me.SelectedMovie.Video.FullPath)
                'Preview.SetPicture(MPEGDecoder.MPEG2Video.DecodePicture(Me.SelectedStream.Video, BytePos / 2))
                Preview.ShowDialog()
            End If
        End Sub

        Private Sub StreamTree_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles MyBase.KeyDown
            Select Case e.KeyData
                Case Keys.Delete
                    Call Me.DeleteSelected()
            End Select
        End Sub
#End Region

#Region " Kontextmen Ereignisse"
        Private Sub StreamTree_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MyBase.MouseDown
            If e.Button = MouseButtons.Right Then
                Dim node As TreeNode = Me.GetNodeAt(Me.PointToClient(Me.MousePosition))
                If Not node Is Nothing Then
                    Me.SelectedNode = node
                    If TypeOf (Me.SelectedNode) Is UI.AudioNode Then
                        Me.ContextMenu = Me.ContextAudio
                    ElseIf TypeOf Me.SelectedNode Is UI.VideoNode Then
                        Me.ContextMenu = Me.ContextVideo
                    ElseIf TypeOf Me.SelectedNode Is UI.StreamNode Then
                        Me.ContextMenu = Me.ContextStream
                    Else
                        Me.ContextMenu = Me.ContextNone
                    End If
                Else
                    Me.ContextMenu = Me.ContextNone
                End If
            End If
        End Sub

        Private Sub mniRenameStream_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mniRenameStream.Click
            Dim RenameDialog As New UI.EditTextDialog

            RenameDialog.EditText = Me.SelectedMovie.Text

            If RenameDialog.ShowDialog = DialogResult.OK Then
                Me.SelectedMovie.Text = RenameDialog.EditText
                Me.SelectedNode.Text = RenameDialog.EditText
            End If
        End Sub

        Private Sub mniAddVideo_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mniAddVideo.Click
            Me.AddVideo()
        End Sub

        Private Sub mniAddAudio_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mniAddAudio.Click
            Me.AddAudio()
        End Sub

        Private Sub Del_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mniDelStream.Click, mniDelVideo.Click, mniDelAudio.Click
            Call Me.DeleteSelected()
        End Sub

        Private Sub mniAudioLang_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mniAudioLang.Click
            Dim AudioNode As UI.AudioNode = DirectCast(Me.SelectedNode, AudioNode)
            Dim dlgLanguage As New UI.LanguageDialog
            dlgLanguage.SelectedCode = AudioNode.Audio.Language

            If dlgLanguage.ShowDialog() = DialogResult.OK Then
                AudioNode.Audio.Language = dlgLanguage.SelectedCode
                AudioNode.Text = AudioNode.Audio.ToString
            End If
        End Sub
#End Region
    End Class

#Region " Benutzerdefinierte TreeNodes"
    'Knoten im MovieTree, der einen Film darstellt
    Public Class StreamNode
        Inherits TreeNode

        Public Movie As New Muxxi.Data.Movie

        Public Sub New()
            MyBase.New()
        End Sub

        Public Sub New(ByVal Text As String)
            Me.Text = Text
            Me.ImageIndex = Images.Icon.Input
            Me.SelectedImageIndex = Images.Icon.Input
            Me.BackColor = Color.Azure
        End Sub

        Public Sub New(ByVal Movie As Muxxi.Data.Movie)
            Me.Movie = Movie
            Me.Text = Movie.Text
            Me.ImageIndex = Images.Icon.Input
            Me.SelectedImageIndex = Images.Icon.Input
            Repaint()
        End Sub

        Private Sub Repaint()
            Me.Nodes.Clear()

            If Not Me.Movie.Video.FullPath = String.Empty Then
                Me.Nodes.Add(New UI.VideoNode(Me.Movie.Video.FileName))
            End If

            Dim Audio As Muxxi.Data.Audio
            For Each Audio In Me.Movie.Audios()
                Me.Nodes.Add(New UI.AudioNode(Audio))
            Next
            Me.ExpandAll()
        End Sub
    End Class

    'Knoten im MovieTree, der eine Video-Datei darstellt
    Public Class VideoNode
        Inherits TreeNode

        Public Sub New()
            MyBase.New()
        End Sub

        Public Sub New(ByVal Video As String)
            Me.Text = Video
            Me.ImageIndex = Images.Icon.Video
            Me.SelectedImageIndex = Images.Icon.Video
        End Sub
    End Class

    'Knoten im MovieTree, der eine Audio-Datei darstellt
    Public Class AudioNode
        Inherits TreeNode

        Public Audio As Data.Audio

        Public Sub New()
            MyBase.New()
        End Sub

        Public Sub New(ByVal Audio As Muxxi.Data.Audio)
            Me.Audio = Audio
            Me.Text = Audio.ToString
            Me.ImageIndex = Images.Icon.Audio
            Me.SelectedImageIndex = Images.Icon.Audio
        End Sub
    End Class
#End Region
End Namespace
